/* Last updated 2/4/98 by TSHII */

/********************** MATLAB.TM ************************************/

#include <stdlib.h>
#include <string.h>
#include "mathlink.h"
#include "engine.h"
#include "mat.h"

/********************* Fix for 3.0 MathLink **************************/
/** stolen code from FastBinaryFiles to get around an mprep bug      */
/** be careful of side effects if you really want to use MLPutString */
/** instead of MLPutByteString                                       */
/*********************************************************************/

#if defined(MLVERSION) && (MLVERSION >= 3)
#       define MLPutString(ml, str) MLPutByteString(ml, str, strlen(str))
#       define MLGetString(ml, str) MLGetByteString(ml, str, &mlgettmp, 255)
#       define MLDisownString(ml, str) MLDisownByteString(ml, str, mlgettmp)
long mlgettmp;
#endif /* MLVERSION */

/*******************  Package Code ***********************************/

:Evaluate:  BeginPackage["Matlab`"]

:Evaluate: MatNames  ::usage   = "MatNames[\"filename\"] returns a list containing the names of the matrices stored in the MATLAB MAT-file \"filename\"."
:Evaluate: MatRead   ::usage   = "MatRead[\"filename\", \"matrixname\"] returns the named matrix from the MATLAB MAT-file \"filename\"."
:Evaluate: MatWrite  ::usage   = "MatWrite[\"filename\", \"matrixname\", mat] writes the Mathematica matrix mat into a MATLAB MAT-file. It writes into the file \"filename\" and calls the matrix \"matrixname\"."
:Evaluate: MatUpdate ::usage   = "MatUpdate[\"filename\", \"matrixname\", mat] updates the Mathematica matrix mat into a previously existing MATLAB MAT-file. It writes into the file 'filename' and calls the matrix \"matrixname\"."
:Evaluate: MatQ      ::usage   = "MatQ[x] returns True if x is expressible as a MATLAB matrix."
:Evaluate: MatWrite  ::badform = "Third argument to MATLABWriteMatrix not expressible as a MATLAB array."

:Evaluate: MOpen     ::usage   = "MOpen[] starts the Matlab Engine and connects to it."
:Evaluate: MClose    ::usage   = "MClose[] closes the connection to the Matlab Engine."
:Evaluate: MGetMatrix::usage   = "MGetMatrix[ \"name\" ] retrieves name from the Matlab workspace and makes it an element in the Mathematica workspace."
:Evaluate: MPutMatrix::usage   = "MPutMatrix[ \"name\", mat ] takes the Mathematica matrix, mat, and translates it to a Matlab object under name. This will only work if mat can evaluate to a numerical matrix."
:Evaluate: MEvaluate ::usage   = "MEvaluate[ \"cmd\", buffer ] runs the comand; cmd verbatim in the Matlab workspace and returns a string with the Matlab response.  The variable buffer is optional and limits the size of the string which is returned to Mathematica."
:Evaluate: M         ::usage   = "M[ \"cmd\", buffer ] short for MEvaluate."
:Evaluate: MBegin    ::usage   = "MBegin[] this takes evaluates every string in Matlab. Basically we subsitute Mathematica'sevaluator for Matlabs. The string notation is kept so that we can escape and still send commands to Mathematica's evaluator. To end this environment use MEnd[]."
:Evaluate: MEnd      ::usage   = "MEnd[] this terminates the Matlab environment started by MBegin[]."

:Evaluate: MPut      ::usage   = ""
:Evaluate: MGet      ::usage   = ""

:Evaluate: MERROR    ::matlaberr = "Matlab error; `1`."

:Evaluate: Begin["`Private`"]

:Evaluate: MatQ[x_]:=VectorQ[x,NumberQ[N[#]]&]||MatrixQ[x,NumberQ[N[#]]&]

/****  Mathematica Symbolic Toolbox for MATLAB functionality *********/

:Begin:
:Function:       MatNames
:Pattern:        MatNames[filename_String]
:Arguments:      { ToLowerCase[filename], ToLowerCase[Directory[]] }
:ArgumentTypes:  { String, Manual }
:ReturnType:     Manual
:End:

:Begin:
:Function:       MatRead
:Pattern:        MatRead[ filename_String, matrixname_String ]
:Arguments:      { ToLowerCase[filename], matrixname, ToLowerCase[Directory[]] }
:ArgumentTypes:  { String, String, Manual }
:ReturnType:     Manual
:End:

:Begin:
:Function:       IntMatWriteOrUpdate
:Pattern:        IntMatWriteOrUpdate[ filename_String, matrixname_String, writeupdate_String, mat_List ]
:Arguments:      { ToLowerCase[filename], matrixname, writeupdate, N[mat], ToLowerCase[Directory[]] }
:ArgumentTypes:  { String, String, String, Manual }
:ReturnType:     Manual
:End:

:Evaluate: MatWrite[filename_String, matrixname_String, mat_List]  := IntMatWriteOrUpdate[filename, matrixname, "w", mat]
:Evaluate: MatUpdate[filename_String, matrixname_String, mat_List] := IntMatWriteOrUpdate[filename, matrixname, "u", mat]

:Evaluate: MatWrite[x_] := (Message[MATLABWriteMatrix::badform];$Failed)

/********************  MAMA functionality ****************************/

:Begin:
:Function:       MOpen
:Pattern:        MOpen[ ]
:Arguments:      { }
:ArgumentTypes:  { }
:ReturnType:     Manual
:End:

:Begin:
:Function:       MClose
:Pattern:        MClose[]
:Arguments:      { }
:ArgumentTypes:  { }
:ReturnType:     Manual
:End:

:Begin:
:Function:       IntMEvaluate
:Pattern:        IntMEvaluate[ cmd_String, buffer_Integer:1000 ]
:Arguments:      { cmd, buffer }
:ArgumentTypes:  { String, Integer }
:ReturnType:     Manual
:End:

:Begin:
:Function:       IntMGetMatrix
:Pattern:        IntMGetMatrix[ name_String ]
:Arguments:      { name }
:ArgumentTypes:  { String }
:ReturnType:     Manual
:End:

:Begin:
:Function:       IntMPutMatrix
:Pattern:        IntMPutMatrix[ name_String, a_List ]
:Arguments:      {name, a}
:ArgumentTypes:  {String, Manual}
:ReturnType:     Manual
:End:

:Evaluate: matlabIn[ x_ ]        := Evaluate[ x ]
:Evaluate: matlabIn[ x_String ]  := MEvaluate[ x ]

:Evaluate: matlabOut[ Null ]      = Null
:Evaluate: matlabOut[ x_String ] := StringReplace[ x, {">> \n" -> "", ">> " -> ""} ]

:Evaluate: MBegin[] := (Unprotect[ In, Out ]; Format[ In  ] = "MatlabIn"; Format[ Out ] = "MatlabOut"; Protect[ In, Out ]; $Pre = matlabIn; $Post = matlabOut; Null);
:Evaluate: MEnd[]   := ($Pre =.; $Post =.; Unprotect[ In, Out ]; Format[ In  ] = "In"; Format[ Out ] = "Out"; Protect[ In, Out ]; Null )

:Evaluate: StrDropLast[x_String] := StringDrop[x, -1];
:Evaluate: StrDropFirst[x_String] := StringDrop[x, 1];

:Evaluate: MPut[name_String, x:(_Real | _Integer | _String)] := M@StringJoin@{name, "=", ToString@x};
:Evaluate: MPut[name_String, x_List] := Module[{dim}, IntMPutMatrix[name, Flatten@Transpose[x, Reverse@Range@Length@Dimensions@x]]; dim = If[Length@Dimensions@x == 1, Append[Dimensions@x, 1], Dimensions@x]; M@StringJoin@{name, " = reshape(", name, ", ", StrDropLast@StrDropFirst@ToString@dim, ")"} ]
:Evaluate: MGet[name_String]         := Module[{mgpdims, mgpm}, M@StringJoin@{"mgpdims = size(", name, ")"}; mgpdims = Reverse@Floor@First@IntMGetMatrix@"mgpdims"; M@StringJoin@{"mgpm = reshape(", name, ",1,", ToString@Floor@(Times@@mgpdims), ")"}; mgpm = First@Matlab`Private`IntMGetMatrix@"mgpm"; M@"clear mgpdims mgpm"; If[Floor@(Times@@mgpdims)== 1, First@mgpm, Transpose[First@Fold[Partition, mgpm,  Reverse@mgpdims], Reverse@Range@Length@mgpdims]] ]

:Evaluate: MEvaluate[command_String, buffer___Integer] := Module[{str, len}, str = IntMEvaluate[command, buffer]; If[StringLength[str] > 0, StringDrop[StringDrop[str, 1], -2], str]]
:Evaluate: M[command_String, buffer___Integer]         := MEvaluate[command, buffer]

:Evaluate: End[]
:Evaluate: EndPackage[]

/********************* Global Variables ******************************/

/* used to send messages via MathLink */
#define ERRBUFSIZE       200
#define MAXMESSAGEPARTS  4
static char              *msgv[MAXMESSAGEPARTS];

/* The standard communication medium */

Engine                   *ep; 

/***************************  C Code  ********************************/

/***********************  MAMA functions  ****************************/

/*****************  send_message_to_mathematica **********************/

static int send_message_to_mathematica(char *symb, char *tag, int msgc, char *msgv[])
    {
    int pkt,i;
    
    MLPutFunction( stdlink, "EvaluatePacket", 1);
    MLPutFunction( stdlink, "Message", msgc+1);
    MLPutFunction( stdlink, "MessageName", 2);
    MLPutSymbol( stdlink, symb);
    MLPutString( stdlink, tag);
    for (i = 0; i < msgc; i++) MLPutString( stdlink, msgv[i]);
    MLEndPacket( stdlink);
    
    while( (pkt = MLNextPacket( stdlink)) && pkt != RETURNPKT)
        MLNewPacket( stdlink);
    MLNewPacket( stdlink);
    
    return MLError( stdlink) == MLEOK;
    }

/***********************  matlaberror ********************************/
/* Standard error handler                                            */
/* This handler uses MathLink to                                     */
/* return its error message. It also returns $Failed.                */
/*********************************************************************/

void matlaberror(char *error_text)
    {
    char errBuf[ERRBUFSIZE];
    sprintf(errBuf, "%s",error_text);
    msgv[0] = errBuf;
    send_message_to_mathematica("MERROR", "matlaberr", 1, msgv);
    }

/*********************************************************************/

void MOpen(void)
    {
    if(ep) 
        {
        matlaberror("Already Connected to Matlab Engine");
        MLPutSymbol(stdlink, "$Failed");
        return;
        }

    if (ep = engOpen(NULL))
        MLPutSymbol(stdlink, "Null");
    else
        MLPutSymbol(stdlink, "$Failed");
    }

/*********************************************************************/

void MClose(void)
    {
    if(!ep) 
        {
        matlaberror("Not Connected to Matlab Engine");
        MLPutSymbol(stdlink, "$Failed");
        return;
        }

    engClose(ep);
	ep = NULL;
    MLPutSymbol(stdlink, "Null");
    }

/*********************************************************************/

void IntMEvaluate(const char* str, int buffer)
    {
    char*  result = (char*) malloc(buffer * sizeof(char));

    if(!ep) 
        {
        matlaberror("Not Connected to Matlab Engine");
        MLPutSymbol(stdlink, "$Failed");
        return;
        }

    engOutputBuffer(ep, result, buffer );    
    engEvalString(ep, str );

	MLPutString(stdlink, result);
	free(result);
    }

/*********************************************************************/

void IntMGetMatrix(const char* name)
    {
    mxArray   *mp;
    double    *pr, *pi;

    double    *dMLp;
    double    *a;
    long      d;
    long      *dims;
    char      **heads;

    char      list[]    = "List";
    char      complex[] = "Complex";

    const int *tmpdims;
    int       tdims, i;

    if(!ep) 
        {
        matlaberror("Not Connected to Matlab Engine");
        MLPutSymbol(stdlink, "$Failed");
        return;
        }

    mp = engGetArray(ep, name);

    if(mp == NULL)
        {
        MLPutSymbol(stdlink, "engGetArray failed in MatlabGetMatrix");
        return;
        }

    d       = (long  )mxGetNumberOfDimensions(mp);
    dims    = (long *)malloc((d + 1) * sizeof(long));
    if(dims == NULL)
        {
        MLPutString(stdlink, "Malloc failed in MGetMatrix");
        return;
        }

    tmpdims = mxGetDimensions(mp);
    for(i = 0; i < d; i++)
        dims[i] = (long)tmpdims[i];
    tdims = 1;
    for(i = 0; i < d; i++)
        tdims *= dims[i];

    heads = NULL;

    if(mxIsComplex(mp))
        {
		d++;
		dims[d - 1] = 2;
        a = (double *)malloc(2 * tdims * sizeof(double));

        pr = mxGetPr(mp);
        pi = mxGetPi(mp);

        dMLp = a;
        for(i = 0; i < tdims; i++)
            {
            *(dMLp++) = pr[i];
            *(dMLp++) = pi[i];
            }

        heads = (char **)malloc(d * sizeof(char**));
        for(i = 0; i < d - 1; i++)
            {
            heads[i] = (char *)malloc((strlen(list) + 1) * sizeof(char));
            strcpy(heads[i], list);
            }
        heads[i] = (char *)malloc((strlen(complex) + 1) * sizeof(char));
        strcpy(heads[i], complex);

        MLPutRealArray(stdlink, a, dims, heads, d);
        }
    else
        {
        a = (double *)malloc(tdims * sizeof(double));

        pr = mxGetPr(mp);

        dMLp = a;
        for(i = 0; i < tdims; i++)
            a[i] = pr[i];

        MLPutRealArray(stdlink, a, dims, NULL, d);
        }

    free(a);
    free(dims);
    if(heads != NULL)
        {
        for(i = 0; i < d; i++)
            free(heads[i]);
        free(heads);
        }
    mxDestroyArray(mp);
    }

/*********************************************************************/

mxArray *MCreateArray(const char *name, long d, long dims[], double *a, char *complexity)
    {
    mxArray *mp;      /* vars for Matlab */
    double  *pr, *pi;

    double  *dMLp;

    int     tdims, i; /* vars extra */
    
    if(strcmp(complexity, "complex") == 0)
        {
        mp = mxCreateNumericArray(d, (const int *)dims, mxDOUBLE_CLASS, mxCOMPLEX);
        if(mp == NULL)
            return(NULL);

        mxSetName(mp, name);

        pr = mxGetPr(mp);
        pi = mxGetPi(mp);

        tdims = 1;
        for(i = 0; i < d - 1; i++)
            tdims *= dims[i];


        dMLp = a;
        for(i = 0; i < tdims; i++)
            {
            pr[i] = *(dMLp++);
            pi[i] = *(dMLp++);
            }
        }
    else
        {
        mp = mxCreateNumericArray(d, (const int *)dims, mxDOUBLE_CLASS, mxREAL);
        if(mp == NULL)
            return(NULL);

        mxSetName(mp, name);

        pr = mxGetPr(mp);

        tdims = 1;
        for(i = 0; i < d; i++)
            tdims *= dims[i];

        if(memcpy(pr, a, tdims * sizeof(double)) == NULL)
            return(NULL);
        }
    return(mp);
    }

/*********************************************************************/

void IntMPutMatrix(const char *name)
    {
    mxArray *mp;      /* vars for Matlab */

    double  *a;       /* vars for MathLink */
    long    *dims;
    char    **heads;
    long    d;

    long    newdims[2];

    if(!ep) 
        {
        matlaberror("Not Connected to Matlab Engine");
        MLPutSymbol(stdlink, "$Failed");
        return;
        }

    MLGetRealArray(stdlink, &a, &dims, &heads, &d);

    switch(d)
        {
        case(1):
            newdims[0] = dims[0];
            newdims[1] = 1;
            mp = MCreateArray(name, d + 1, newdims, a, "real");
            break;
        case(2):
            if(strcmp(heads[d - 1], "Complex") == 0)
                {
                newdims[0] = dims[0];
                newdims[1] = 1;
                mp = MCreateArray(name, d, newdims, a, "complex");
                }
            else
                mp = MCreateArray(name, d, dims, a, "real");
            break;
        case(3):
            if(strcmp(heads[d - 1], "Complex") == 0)
                {
                newdims[0] = dims[0];
                newdims[1] = dims[1];
                mp = MCreateArray(name, d - 1, newdims, a, "complex");
                }
            else
                {
                MLPutString(stdlink, "dimensions >=3 are not supported");
                return;
                }
            break;
        default:
            MLPutString(stdlink, "dimensions >=3 are not supported");
            return;
            break;
        }

    if(mp == NULL)
        {
        MLPutString(stdlink, "MCreateArray failed in MPutMatrix");
        }

    if(engPutArray(ep, mp) == 1)
        {
        MLPutString(stdlink, "engPutArray failed in MPutMatrix");
        return;
        }

    MLDisownRealArray(stdlink, a, dims, heads, d);

    MLPutSymbol(stdlink, "Null");
    }

/******  Mathematica Symbolic Toolbox for MATLAB functionality  ******/

/***********************  MATLABMatrixNames **************************/

void MatNames(const char *filename)
    {
    MATFile *fp;
    char    **dir;
    int     ndir, i;

    const char    *currentdirectory;

    MLGetString(stdlink, &currentdirectory);

    SetCurrentDirectory(currentdirectory);
	fp = matOpen(filename, "r");
    if (fp == NULL) 
        {
        matlaberror("Can't open mat file for reading");
        MLPutSymbol(stdlink, "$Failed");
        return;
        }

    dir = matGetDir(fp, &ndir);
    matClose(fp);

    if (dir == NULL) 
        {
        matlaberror("Can't read directory");
        MLPutSymbol(stdlink, "$Failed");
        return;
        } 
    else 
        {
        MLPutFunction(stdlink, "List", ndir);
        for (i=0; i < ndir; i++) 
            {
            MLPutString(stdlink, dir[i]);
              }
        }

	MLDisownString(stdlink, currentdirectory);
    mxFree(dir);
    return;
    }

/***********************  MATLABReadMatrix ***************************/

void MatRead(const char *filename, const char *matrixname)
    {
    MATFile *fp;
    mxArray *mp;

    double  *data, *currPtr, *realPtr, *imagPtr;
    long    dims[3];
    char    *heads[3];
    long    depth;
    int     row, col;

    const char    *currentdirectory;

    dims[0] = dims[1] = dims[2] = 0;
    heads[0] = "List"; heads[1] = "List"; heads[2] = "Complex";

    MLGetString(stdlink, &currentdirectory);

    SetCurrentDirectory(currentdirectory);
	fp = matOpen(filename, "r");
    if (fp == NULL) 
        {
        matlaberror("Can't open mat file for reading");
        MLPutSymbol(stdlink, "$Failed");
        return;
        }

    mp = matGetArray(fp, matrixname);
    matClose(fp);

    if(mp == NULL)
        {
        matlaberror("Can't find the requested matrix");
        mxDestroyArray(mp);
        MLPutSymbol(stdlink, "$Failed");
        return;
        }
    else if(mxIsSparse(mp))
        {
        matlaberror("The requested matrix is sparse");
        mxDestroyArray(mp);
        MLPutSymbol(stdlink, "$Failed");
        return;
        }
    else if(mxIsChar(mp))
        {
        matlaberror("The requested matrix is a string");
        mxDestroyArray(mp);
        MLPutSymbol(stdlink, "$Failed");
        return;
        }
    else if(mxIsComplex(mp)) /* it's complex */
        {
        dims[0] = mxGetM(mp);
        dims[1] = mxGetN(mp);
        dims[2] = 2;
        depth   = 3;
        
        data = (double *)malloc(dims[0] * dims[1] * dims[2] * sizeof(double));
        
        if (!data)
            {
            matlaberror("Memory allocation failed in MATLABReadMatrix");
            mxDestroyArray(mp);
            MLPutSymbol(stdlink, "$Failed");
            return;
            }
        
        realPtr = mxGetPr(mp);
        imagPtr = mxGetPi(mp);

        for (currPtr = data, row = 0; row < dims[0]; row++)
            for (col = 0; col < dims[1]; col++) 
                {
                *(currPtr++) = realPtr[col*dims[0]+row];
                *(currPtr++) = imagPtr[col*dims[0]+row];
                }
        }
    else /* it's real */
        {
        dims[0] = mxGetM(mp);
        dims[1] = mxGetN(mp);
        dims[2] = 2;
        depth   = 2;
        
        data = (double *)malloc(dims[0] * dims[1] * sizeof(double));
        
        if (!data)
            {
            matlaberror("Memory allocation failed in MATLABReadMatrix");
            mxDestroyArray(mp);
            MLPutSymbol(stdlink, "$Failed");
            return;
            }
        
        realPtr = mxGetPr(mp);
        
        for (currPtr = data, row = 0; row < dims[0]; row++)
            for (col = 0; col < dims[1]; col++)
                *(currPtr++) = realPtr[col*dims[0]+row];
        }

    MLPutDoubleArray(stdlink, data, dims, heads, depth);
	MLDisownString(stdlink, currentdirectory);
    free(data);
    mxDestroyArray(mp);
    }

/***********************  MATLABWriteMatrix **************************/
    
void IntMatWriteOrUpdate(const char *filename, const char *matrixname, const char *writeupdate)
    {
    MATFile  *fp;
    mxArray  *mp;
    int      mlres;
    double   *data, *currPtr, *realPtr, *imagPtr;
    long     *dims;
    char     **heads;
    long     depth;
    int      row, col;

    const char    *currentdirectory;

//THIS LINE CURRENTLY FAILS DUE TO A BUG IN THE 3.5 KERNEL, IT WORKS FOR 3.0
    mlres = MLGetDoubleArray(stdlink, &data, &dims, &heads, &depth);
    if (!mlres) 
        {
        matlaberror("MLGetDoubleArray failed in MATLABWriteMatrix");
          MLPutSymbol(stdlink, "$Failed");
        return;
        }

    mlres = MLGetString(stdlink, &currentdirectory);
    if (!mlres) 
        {
        matlaberror("MLGetString failed in MATLABWriteMatrix");
        MLPutSymbol(stdlink, "$Failed");
        return;
        }

    SetCurrentDirectory(currentdirectory);
	fp = matOpen(filename, writeupdate);
    if (fp == NULL) 
        {
        matlaberror("Can't open mat file for write");
        MLPutSymbol(stdlink, "$Failed");
        return;
        }

    switch(depth)
        {
        case(1): /* 1D real */
            {
            mp = mxCreateDoubleMatrix(1, dims[0], mxREAL);
            if(mp == NULL)
                {
                matlaberror("Couldn't allocate matrix in MATLABWriteMatrix");
                MLDisownDoubleArray(stdlink, data, dims, heads, depth);
                matClose(fp);
                MLPutSymbol(stdlink, "$Failed");
                return;
                }
            else
                {
                realPtr = mxGetPr(mp);
                for (currPtr = data, col = 0; col < dims[0]; col++)
                    realPtr[col] = *(currPtr++);
                }
            }
        break;
        
        case(2):/* 1D complex or 2D real */
            {
            if (!strcmp(heads[1], "Complex")) /* 1D comples */
                {
                mp = mxCreateDoubleMatrix(1, dims[0], mxCOMPLEX);
                if(mp == NULL)
                    {
                    matlaberror("Couldn't allocate matrix in MATLABWriteMatrix");
                    MLDisownDoubleArray(stdlink, data, dims, heads, depth);
                    matClose(fp);
                    MLPutSymbol(stdlink, "$Failed");
                    return;
                    }
                else
                    {
                    realPtr = mxGetPr(mp);
                    imagPtr = mxGetPi(mp);
                    for (currPtr = data, col = 0; col < dims[0]; col++) 
                        {
                        realPtr[col] = *(currPtr++);
                        imagPtr[col] = *(currPtr++);
                        }
                    }
                }
            else   /* 2D real */
                {
                mp = mxCreateDoubleMatrix(dims[0], dims[1], mxREAL);
                if(mp == NULL)
                    {
                    matlaberror("Couldn't allocate matrix in MATLABWriteMatrix");
                    MLDisownDoubleArray(stdlink, data, dims, heads, depth);
                    matClose(fp);
                    MLPutSymbol(stdlink, "$Failed");
                    return;
                    }
                else
                    {
                    realPtr = mxGetPr(mp);
                    for (currPtr = data, row = 0; row < dims[0]; row++)
                        {
                        for (col = 0; col < dims[1]; col++)
                            {
                            realPtr[col*dims[0]+row] = *(currPtr++);
                            }
                        }
                    }
                }
            }
            break;

        case(3): /* 2D Complex */
            {
            if (!strcmp(heads[2], "Complex"))
                {
                mp = mxCreateDoubleMatrix(dims[0], dims[1], mxCOMPLEX);
                if(mp == NULL)
                    {
                    matlaberror("Couldn't allocate matrix in MATLABWriteMatrix");
                    MLDisownDoubleArray(stdlink, data, dims, heads, depth);
                    matClose(fp);
                    MLPutSymbol(stdlink, "$Failed");
                    return;
                    }
                else
                    {
                    realPtr = mxGetPr(mp);
                    imagPtr = mxGetPi(mp);
                    for (currPtr = data, row = 0; row < dims[0]; row++)
                        {
                        for (col = 0; col < dims[1]; col++) 
                            {
                            realPtr[col*dims[0]+row] = *(currPtr++);
                            imagPtr[col*dims[0]+row] = *(currPtr++);
                            }
                        }
                    }
                }
            else
                {
                matlaberror("MATLABIO can handle only 1D or 2D arrays");
                MLDisownDoubleArray(stdlink, data, dims, heads, depth);
                matClose(fp);
                MLPutSymbol(stdlink, "$Failed");
                return;
                }
            }
            break;

        default:
            {
            matlaberror("MATLAB can handle only 1D or 2D arrays");
            MLDisownDoubleArray(stdlink, data, dims, heads, depth);
            matClose(fp);
              MLPutSymbol(stdlink, "$Failed");
            return;
            }
        }
    MLDisownDoubleArray(stdlink, data, dims, heads, depth);
	MLDisownString(stdlink, currentdirectory);

    mxSetName(mp, matrixname);
    matPutArray(fp, mp);
    mxDestroyArray(mp);
    matClose(fp);
    MLPutString(stdlink, filename);
    return;
    }

/*****************  Platform Dependent Main Functions ****************/

#if MACINTOSH_MATHLINK

int main( int argc, char* argv[])
    {
    /* Due to a bug in some standard C libraries that have shipped with
     * MPW, zero is passed to MLMain below.  (If you build this program
     * as an MPW tool, you can change the zero to argc.)
     */
    argc = argc; /* suppress warning */
    return MLMain( 0, argv);
    }

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HANDLE hinstCurrent, HANDLE hinstPrevious, LPSTR lpszCmdLine, int nCmdShow)
    {
    char  buff[512];
    char FAR * buff_start = buff;
    char FAR * argv[32];
    char FAR * FAR * argv_end = argv + 32;

    if( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
    MLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
    return MLMain( argv_end - argv, argv);
    }

#else

int main(argc, argv)
    int argc; char* argv[];
    {
    return MLMain(argc, argv);
    }

#endif

/*********************************************************************/
