(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     57919,       2049]*)
(*NotebookOutlinePosition[     71345,       2458]*)
(*  CellTagsIndexPosition[     71161,       2448]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Matlab Migration Packages and Programs", "Title",
  CellTags->"Introduction"],

Cell["\<\
by Tom Hildrich
Original Code by Roger Germundsson and Doug Stein  \
\>", "Text",
  CellTags->"Introduction"],

Cell[TextData[{
  "This package is composed of two parts.  The first allows you to read and \
write Matlab MAT-Files.  The second part allows you read and write \
information directly between the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Kernel and the Matlab Engine.  Both parts were written by combining the \
Matlab API and ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  ", ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s API.  You can find out mor about ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " at the following URL:\n\n",
  Cell[BoxData[
      ButtonBox[
        \(\(http : \) // \(\(www . wolfram . com/support\)/MathLink/\)\),
        ButtonData:>{
          URL[ "http://www.wolfram.com/support/MathLink/"], None},
        ButtonStyle->"Hyperlink"]], "Input",
    CellTags->"Introduction"]
}], "Text",
  CellTags->"Introduction"],

Cell[CellGroupData[{

Cell["Installation", "Subtitle",
  CellTags->"Installation"],

Cell[CellGroupData[{

Cell["Finding Matlab's Library Files", "Section",
  CellTags->"Installation"],

Cell["\<\
In order for these programs to work correctly, they directory structure \
created when you decompressed them must be placed in the AddOns/Applications \
directory or you must change all of the code that deals with these \
directories to your preferred directory.  It may also be necessary to move \
the following files from your Matlab directory to the \
/Addons/Applications/Matlab/Bin directory:\
\>", "Text",
  CellTags->"Installation"],

Cell["\<\
libeng.dll
libmat.dll
libmx.dll
libut.dll\
\>", "Text",
  FontWeight->"Bold",
  CellTags->"Installation"],

Cell["\<\
Alternatively, you can add the Matlab Bin directory to your path (e.g. \
C:\\Program Files\\Matlab\\Bin).  This will require a reboot, but save hard \
drive space.\
\>", "Text",
  CellTags->"Installation"],

Cell["\<\
It may be necessary to perform a similar operation under UNIX.  Move the lib \
files (.so) to the /usr/lib directory so that they can be found by the \
program at run-time. \
\>", "Text",
  CellTags->"Installation"],

Cell["Read the readme.txt for further installation information.", "Text",
  CellTags->"Installation"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Getting the proper Directory for the Executables and Examples\
\>", "Section",
  CellTags->"Installation"],

Cell[TextData[{
  "If you change the installation directory to something other than ",
  StyleBox["AddOns/Applications",
    FontWeight->"Bold"],
  ",",
  StyleBox[" ",
    FontWeight->"Bold"],
  "currently the default, then you will need to change the package code to \
mirror this.  "
}], "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Installing the Package and Shutting Down", "Section",
  Evaluatable->False,
  CellTags->"Installation"],

Cell["First, we need to load the package and executable.", "Text",
  CellTags->"Installation"],

Cell[BoxData[
    \(<< Matlab`Matlab`\)], "Input",
  CellTags->"Installation"],

Cell[TextData[{
  StyleBox[" This package will properly quit when the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[
  " kernel quits. Nothing special needs to be done. If, however, you wish to \
manually quit without quitting ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[", simply evaluate:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  CellTags->"Installation"],

Cell["MatlabUninstall[];", "Input",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Commands", "Section",
  CellTags->"Installation"],

Cell[CellGroupData[{

Cell["MAT-File Commands", "Subsection",
  CellTags->"Installation"],

Cell[CellGroupData[{

Cell["MatNames[\"filename\"]", "Subsubsection",
  CellTags->"Installation"],

Cell["\<\
Returns a list containing all the names of matrices stored in the Matlab \
MAT-file \"filename\".\
\>", "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MatRead[\"filename\", matrixname]", "Subsubsection",
  CellTags->"Installation"],

Cell["\<\
Returns the named matrix from the Matlab MAT-File named \"filename\".\
\>", "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MatWrite[\"filename\", matrixname, mat]", "Subsubsection",
  CellTags->"Installation"],

Cell["\<\
Writes mat as matrix name to the Matlab MAT-File named \"filename\".  This \
function will wipe out the contents of an already created file or create a \
new file if one doesn't already exist.\
\>", "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MatUpdate[\"filename\", matrixname, mat]", "Subsubsection",
  CellTags->"Installation"],

Cell["\<\
Updates the Matlab MAT-File named \"filename\" with a new mat named \
matrixname.  This function can only use a file that already exists.  If it \
doesn't an error will be returned.\
\>", "Text",
  CellTags->"Installation"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Engine Commands", "Subsection",
  CellTags->"Installation"],

Cell[CellGroupData[{

Cell["MOpen[]", "Subsubsection",
  CellTags->"Installation"],

Cell["Open a connection to Matlab process.", "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MClose[]", "Subsubsection",
  CellTags->"Installation"],

Cell["Close a connection to a Matlab process.", "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MEvaluate[\"command\"]", "Subsubsection",
  CellTags->"Installation"],

Cell["Evaluate an expression verbatim in the Matlab environment.", "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MBegin[]", "Subsubsection",
  CellTags->"Installation"],

Cell["\<\
Start a Matlab mode. Every string is evaluated in the Matlab environment.\
\>", "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MEnd[]", "Subsubsection",
  CellTags->"Installation"],

Cell["End the Matlab mode.", "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MPut[\"name\", expr]", "Subsubsection",
  CellTags->"Installation"],

Cell["\<\
This function can handle real numbers, integers and lists.  It translates \
them to the appropriate form in Matlab.  The name given is assigned to this \
new Matlab value.\
\>", "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MGet[\"name\"]", "Subsubsection",
  CellTags->"Installation"],

Cell["\<\
This function gets a value with the name \"name\" from Matlab and turns into \
it into a real number or a list of real numbers.\
\>", "Text",
  CellTags->"Installation"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Reading, Writing and Updating Matlab Mat-Files", "Subtitle",
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell["Read the Penny data", "Section",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[BoxData[
    \(<< Matlab`Matlab`\)], "Input",
  CellTags->"Mat"],

Cell[BoxData[
    \(\(SetDirectory[
      ToFileName[{$TopDirectory, \ "\<AddOns\>", \ "\<Applications\>", \ 
          "\<Matlab\>", "\<Examples\>"}\ ]]; \)\)], "Input",
  CellTags->"Mat"],

Cell[TextData[{
  StyleBox[
  "First, let's see what matrices are in the file. MAT-files allow multiple \
matrices in a file, so ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["MatNames",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" gives the \"table of contents\" of the MAT-file.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatNames["\<Penny.mat\>"]\)], "Input",
  CellTags->"Mat"],

Cell[BoxData[
    \({"P"}\)], "Output",
  CellTags->"Mat"]
}, Open  ]],

Cell["\<\
It seems that the data is descriptively named \"P\". I have to reverse the \
data since the MAT-file has Lincoln upside down. The MATLAB folks take care \
of this by setting a funny plotting option; we can handle this by reversing \
the data.\
\>", "Text",
  CellTags->"Mat"],

Cell["penny = Reverse[MatRead[\"Penny.mat\", \"P\"]];", "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell["Dimensions[penny]", "Input",
  CellTags->"Mat"],

Cell[BoxData[
    \({128, 128}\)], "Output",
  CellTags->"Mat"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plotting the Penny", "Section",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["Here are three obvious visualizations using this data:", "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["ListDensityPlot[penny, Mesh->False];", "Input",
  CellTags->"Mat"],

Cell["ListContourPlot[penny];", "Input",
  CellTags->"Mat"],

Cell["\<\
The ListPlot3D hasn't been computed here since it generates lots of \
PostScript and would make this notebook rather large. You can generate it if \
you want.....\
\>", "SmallText",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["ListPlot3D[penny, Mesh->False];", "Input",
  CellTags->"Mat"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coloring the Penny (Optional)", "Section",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[TextData[{
  StyleBox[
  "Now we'll color the penny the same way as the MATLAB folks. Their formula \
is:\n",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["\tc = min(1,gray(m)*diag([1.2500 0.7812 0.4975]));",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Bold"],
  StyleBox["\nOurs would be:",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["\<\
CopperColor[val_] := RGBColor[
\t\t\t\t\t\tMin[1, 1.25 val],
\t\t\t\t\t\t0.7812 val,
\t\t\t\t\t\t0.4975 val
\t\t\t\t\t]\
\>", "Input",
  CellTags->"Mat"],

Cell[TextData[StyleBox[
"Note: ColorFunction->CopperColor looks good only at 16 or 24-bit color \
(Thousands or Millions in Macintosh lingo). If running at less colors, omit \
this ColorFunction option to get a grey-scale image. Macintosh users can also \
choose \"Colors...\" from the File menu and pick \"Dither patterns\" for the \
dialog item entitled \"When rendering bitmap images (DensityPlots), use:\"",
  Evaluatable->False,
  AspectRatioFixed->True,
  FontWeight->"Bold"]], "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["\<\
ListDensityPlot[penny, Mesh->False,
\tColorFunction->CopperColor];\
\>", "Input",
  CellTags->"Mat"],

Cell["\<\
ListPlot3D[penny, Mesh->False,
\tColorFunction->CopperColor];\
\>", "Input",
  CellTags->"Mat"],

Cell["\<\
ListContourPlot[penny,
\tColorFunction->CopperColor];\
\>", "Input",
  CellTags->"Mat"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Modifying the Penny", "Section",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[TextData[{
  StyleBox[
  "Let's severely muddle the penny data by taking a modified Fourier \
transform. I follow a suggestion developed in the book ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Applied ",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontVariations->{"Underline"->True}],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic",
    FontVariations->{"Underline"->True}],
  StyleBox[" by Shaw and Tigg on page 412. ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["centeredFourier[]",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " shifts the low-frequency components into the center of the picture and \
shifts the high-frequency components to the periphery.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["\<\
centeredFourier[x_List] := \
Transpose[RotateRight[Transpose[RotateRight[Fourier[x], 64]], 64]];\
\>", "Input",
  CellTags->"Mat"],

Cell["pennyFourier = centeredFourier[penny];", "Input",
  CellTags->"Mat"],

Cell["ListDensityPlot[Abs[pennyFourier], Mesh->False];", "Input",
  CellTags->"Mat"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Writing the Modified Data", "Section",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[TextData[{
  StyleBox["Let's write to a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  "Mat-file",
  StyleBox[
  " (PennyFourier.mat) and name as a Matlab matrix (PennyFourier) the ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" matrix (pennyFourier). Got it?",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["MatWrite[\"PennyFourier.mat\", \"PennyFourier\", pennyFourier];", 
  "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatNames["\<PennyFourier.mat\>"]\)], "Input",
  CellTags->"Mat"],

Cell[BoxData[
    \({"PennyFourier"}\)], "Output",
  CellTags->"Mat"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Advanced Uses", "Section",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell["Writing a list of matrices into a single file", "Subsection",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[TextData[{
  StyleBox["If you want to write a list of matrices to a mat file from ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" , simply ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Map",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" the writing function over the list of matrices. The variable ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["matnum",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[
  " is used to give distinct names to the matrices (otherwise the Matlab file \
writing library used by this package will overwrite rather than append \
matrices).",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["\<\
The interface changed a little since 4.x.  Now 2 functions exist for writing \
files MatWrite to write matrices to a file which does not yet exist and \
MatWrite which will update a mat file instead of writing over any contents \
already there.\
\>", "Text",
  CellTags->"Mat"],

Cell["\<\
writeAllMatrices[filename_String, matrices_] :=
\tModule[{name = filename, matnum = 1},
\tMatWrite[name, \"mat\"<>ToString[matnum++], matrices[[1]]];
\tMap[MatUpdate[name, \"mat\"<>ToString[matnum++], #]&, Drop[matrices,1]];
\tname
\t]\
\>", "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["Let's make a list of three matrices.", "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["\<\
matList = {
\tTable[Sin[x y],{x,0,Pi,Pi/10},{y,0,Pi,Pi/10}],
\tTable[Random[],{15},{15}],
\tTable[Mod[x,y],{x,1,10},{y,1,10}]
\t};\
\>", "Input",
  CellTags->"Mat"],

Cell["Checking the dimensions of the matrices...", "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell["Map[Dimensions, matList]", "Input",
  CellTags->"Mat"],

Cell[BoxData[
    \({{11, 11}, {15, 15}, {10, 10}}\)], "Output",
  CellTags->"Mat"]
}, Open  ]],

Cell["Now writing them...", "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["writeAllMatrices[\"TripleThreat.mat\", matList];", "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatNames["\<TripleThreat.mat\>"]\)], "Input",
  CellTags->"Mat"],

Cell[BoxData[
    \({"mat1", "mat2", "mat3"}\)], "Output",
  CellTags->"Mat"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Reading all the matrices in a file", "Subsection",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[TextData[{
  StyleBox["If you want to read all the matrices in a MAT-file into a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" list, simply ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Map",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" the reading function over the list of matrix names.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["\<\
readAllMatrices[filename_String] :=
\tModule[{name = filename},
\t\tMap[MatRead[name, #]&,
\t\t\tMatNames[name]]
\t]\
\>", "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["Let's read the matrices we just wrote...", "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell["newMatList = readAllMatrices[\"TripleThreat.mat\"];", "Input",
  CellTags->"Mat"],

Cell[TextData[{
  StyleBox["...and for fun, let's make a ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["ListDensityPlot",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" of each. I'm using ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["DisplayFunction->Identity",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" since I want to combine the plots into one graphic.",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell["\<\
plots = Map[
\tListDensityPlot[#, DisplayFunction->Identity]&,
\tnewMatList
]\
\>", "Input",
  CellTags->"Mat"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          \(\[SkeletonIndicator]  DensityGraphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[
          \(\[SkeletonIndicator]  DensityGraphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[
          \(\[SkeletonIndicator]  DensityGraphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output",
  CellTags->"Mat"]
}, Open  ]],

Cell[TextData[{
  StyleBox["Now that I have the plots I'll combine them into one ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["GraphicsArray",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell["Show[GraphicsArray[plots]]", "Input",
  CellTags->"Mat"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  GraphicsArray  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"Mat"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Error Handling", "Section",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"Mat"],

Cell[TextData[{
  StyleBox["The functions in this package create ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["Mathematica",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontSlant->"Italic"],
  StyleBox[" warning messages if something goes wrong. They then return ",
    Evaluatable->False,
    AspectRatioFixed->True],
  StyleBox["$Failed",
    Evaluatable->False,
    AspectRatioFixed->True,
    FontWeight->"Bold"],
  StyleBox[".",
    Evaluatable->False,
    AspectRatioFixed->True]
}], "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[CellGroupData[{

Cell["MatNames[\"foobar.mat\"]", "Input",
  CellTags->"Mat"],

Cell[BoxData[
    \(MERROR::"matlaberr" \( : \ \) 
      "Matlab error; \!\(\"Can't open mat file for reading\"\)."\)], "Message",\

  CellTags->"Mat"],

Cell[BoxData[
    \($Failed\)], "Output",
  CellTags->"Mat"]
}, Open  ]],

Cell["MatlabUninstall[];", "Input",
  CellTags->"Mat"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Connecting to the Matlab Engine", "Subtitle",
  CellTags->"M"],

Cell[TextData[{
  "This part of the package solves the reverse problem of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Symbolic Toolbox for MATLAB.  (",
  StyleBox["MathSource",
    FontSlant->"Italic"],
  " item 0205-951) That is, it allows the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " user to call Matlab from within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". It is implemented as a ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " template program to talk to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and uses Matlab's engine interface to talk to Matlab.  However, since ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " can connect heterogeneous platforms, you can call this program from any ",
  
  StyleBox["MathLink",
    FontSlant->"Italic"],
  "-capable copy of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  For example, if you have Matlab on a Sun build it there.  Now you can \
call MaMa from ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " running on any Unix, Macintosh, or Microsoft Windows machine."
}], "Text",
  CellTags->"M"],

Cell[CellGroupData[{

Cell["Example", "Section",
  CellTags->"M"],

Cell[" Load the package", "Text",
  CellTags->"M"],

Cell[BoxData[
    \(<< Matlab`Matlab`\)], "Input",
  CellTags->"M"],

Cell["First, Start the Matlab Engine", "Text",
  CellTags->"M"],

Cell[BoxData[
    \(MOpen[]\)], "Input",
  CellTags->"M"],

Cell["Now you can send commands to Matlab's Engine", "Text",
  CellTags->"M"],

Cell[CellGroupData[{

Cell[BoxData[
    \(MEvaluate["\<[1,2;3,4]\>"]\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("ans =\n\n     1     2\n     3     4"\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "If you're worried that the size of the output will be larger than the \
current maximum of 1000, then you can provide for a larger buffer using ",
  StyleBox["MEvaluate",
    FontWeight->"Bold"],
  " as follows."
}], "Text",
  CellTags->"M"],

Cell[CellGroupData[{

Cell[BoxData[
    \(MEvaluate["\<[1,2;3,4]\>", \ 10000]\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("ans =\n\n     1     2\n     3     4"\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The shortended form of ",
  StyleBox["MEvaluate",
    FontWeight->"Bold"],
  " also allows for an increased buffer."
}], "Text",
  CellTags->"M"],

Cell[CellGroupData[{

Cell[BoxData[
    \(M["\<[1,2;3,4]\>", \ 10000]\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("ans =\n\n     1     2\n     3     4"\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "It is also possible to send a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " expressions to Matlab ...\nLet's send some numeric arrays of arbitrary \
size"
}], "Text",
  CellTags->"M"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\((a = \ Table[\ i\ *\ j, \ {i, 3}, \ {j, 4}\ ])\) // TableForm\), 
    \(\((b = \ Table[x\  + \ x\ I, \ {x, \ 3}, \ {x, \ 4}])\) // TableForm
      \), 
    \(\((c = \ 
          Table[x\ y\ z, \ {x, \ 1, \ 2}, \ {y, \ 1, \ 3}, \ {z, \ 1, \ 4}])
        \) // MatrixForm\), 
    \(\((d\  = \ 
          Table[x\  + \ y\  + \ z\ I, \ {x, \ 1, \ 5}, \ {y, \ 1, \ 4}, 
            \ {z, \ 1, \ 2}])\) // MatrixForm\), 
    \(\((e\  = \ 
          Table[u\ *\ w\ *\ x\ *\ y\ *\ z, {u, \ 2}, {w, \ 3}, \ {x, \ 2}, 
            \ {y, \ 3}, \ {z, \ 2}])\) // MatrixForm\), 
    \(\((f\  = \ Table[i, {i, \ 10}])\) // MatrixForm\), 
    \(\((g\  = \ 10)\) // TableForm\)}], "Input",
  CellTags->"M"],

Cell[BoxData[
    TagBox[GridBox[{
          {"1", "2", "3", "4"},
          {"2", "4", "6", "8"},
          {"3", "6", "9", "12"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"],

Cell[BoxData[
    TagBox[GridBox[{
          {\(1 + I\), \(2 + 2\ I\), \(3 + 3\ I\), \(4 + 4\ I\)},
          {\(1 + I\), \(2 + 2\ I\), \(3 + 3\ I\), \(4 + 4\ I\)},
          {\(1 + I\), \(2 + 2\ I\), \(3 + 3\ I\), \(4 + 4\ I\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"2"},
                    {"3"},
                    {"4"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"2"},
                    {"4"},
                    {"6"},
                    {"8"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"3"},
                    {"6"},
                    {"9"},
                    {"12"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"2"},
                    {"4"},
                    {"6"},
                    {"8"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"4"},
                    {"8"},
                    {"12"},
                    {"16"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"6"},
                    {"12"},
                    {"18"},
                    {"24"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{1, 2, 3, 4}, {2, 4, 6, 8}, {3, 6, 9, 12}}, {{2, 4, 6, 
        8}, {4, 8, 12, 16}, {6, 12, 18, 24}}}]]], "Output"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {\(2 + I\)},
                    {\(2 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(3 + I\)},
                    {\(3 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(4 + I\)},
                    {\(4 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(5 + I\)},
                    {\(5 + 2\ I\)}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {\(3 + I\)},
                    {\(3 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(4 + I\)},
                    {\(4 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(5 + I\)},
                    {\(5 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(6 + I\)},
                    {\(6 + 2\ I\)}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {\(4 + I\)},
                    {\(4 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(5 + I\)},
                    {\(5 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(6 + I\)},
                    {\(6 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(7 + I\)},
                    {\(7 + 2\ I\)}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {\(5 + I\)},
                    {\(5 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(6 + I\)},
                    {\(6 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(7 + I\)},
                    {\(7 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(8 + I\)},
                    {\(8 + 2\ I\)}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {\(6 + I\)},
                    {\(6 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(7 + I\)},
                    {\(7 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(8 + I\)},
                    {\(8 + 2\ I\)}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {\(9 + I\)},
                    {\(9 + 2\ I\)}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{
        Complex[ 2, 1], 
        Complex[ 2, 2]}, {
        Complex[ 3, 1], 
        Complex[ 3, 2]}, {
        Complex[ 4, 1], 
        Complex[ 4, 2]}, {
        Complex[ 5, 1], 
        Complex[ 5, 2]}}, {{
        Complex[ 3, 1], 
        Complex[ 3, 2]}, {
        Complex[ 4, 1], 
        Complex[ 4, 2]}, {
        Complex[ 5, 1], 
        Complex[ 5, 2]}, {
        Complex[ 6, 1], 
        Complex[ 6, 2]}}, {{
        Complex[ 4, 1], 
        Complex[ 4, 2]}, {
        Complex[ 5, 1], 
        Complex[ 5, 2]}, {
        Complex[ 6, 1], 
        Complex[ 6, 2]}, {
        Complex[ 7, 1], 
        Complex[ 7, 2]}}, {{
        Complex[ 5, 1], 
        Complex[ 5, 2]}, {
        Complex[ 6, 1], 
        Complex[ 6, 2]}, {
        Complex[ 7, 1], 
        Complex[ 7, 2]}, {
        Complex[ 8, 1], 
        Complex[ 8, 2]}}, {{
        Complex[ 6, 1], 
        Complex[ 6, 2]}, {
        Complex[ 7, 1], 
        Complex[ 7, 2]}, {
        Complex[ 8, 1], 
        Complex[ 8, 2]}, {
        Complex[ 9, 1], 
        Complex[ 9, 2]}}}]]], "Output"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {
                      RowBox[{"(", GridBox[{
                            {"1"},
                            {"2"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"2"},
                            {"4"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"3"},
                            {"6"}
                            }], ")"}]},
                    {
                      RowBox[{"(", GridBox[{
                            {"2"},
                            {"4"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"4"},
                            {"8"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"6"},
                            {"12"}
                            }], ")"}]}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {
                      RowBox[{"(", GridBox[{
                            {"2"},
                            {"4"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"4"},
                            {"8"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"6"},
                            {"12"}
                            }], ")"}]},
                    {
                      RowBox[{"(", GridBox[{
                            {"4"},
                            {"8"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"8"},
                            {"16"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"12"},
                            {"24"}
                            }], ")"}]}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {
                      RowBox[{"(", GridBox[{
                            {"3"},
                            {"6"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"6"},
                            {"12"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"9"},
                            {"18"}
                            }], ")"}]},
                    {
                      RowBox[{"(", GridBox[{
                            {"6"},
                            {"12"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"12"},
                            {"24"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"18"},
                            {"36"}
                            }], ")"}]}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {
                      RowBox[{"(", GridBox[{
                            {"2"},
                            {"4"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"4"},
                            {"8"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"6"},
                            {"12"}
                            }], ")"}]},
                    {
                      RowBox[{"(", GridBox[{
                            {"4"},
                            {"8"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"8"},
                            {"16"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"12"},
                            {"24"}
                            }], ")"}]}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {
                      RowBox[{"(", GridBox[{
                            {"4"},
                            {"8"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"8"},
                            {"16"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"12"},
                            {"24"}
                            }], ")"}]},
                    {
                      RowBox[{"(", GridBox[{
                            {"8"},
                            {"16"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"16"},
                            {"32"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"24"},
                            {"48"}
                            }], ")"}]}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {
                      RowBox[{"(", GridBox[{
                            {"6"},
                            {"12"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"12"},
                            {"24"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"18"},
                            {"36"}
                            }], ")"}]},
                    {
                      RowBox[{"(", GridBox[{
                            {"12"},
                            {"24"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"24"},
                            {"48"}
                            }], ")"}], 
                      RowBox[{"(", GridBox[{
                            {"36"},
                            {"72"}
                            }], ")"}]}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{{{1, 2}, {2, 4}, {3, 6}}, {{2, 4}, {4, 8}, {6, 12}}}, {{{
        2, 4}, {4, 8}, {6, 12}}, {{4, 8}, {8, 16}, {12, 24}}}, {{{3, 6}, {6, 
        12}, {9, 18}}, {{6, 12}, {12, 24}, {18, 36}}}}, {{{{2, 4}, {4, 8}, {
        6, 12}}, {{4, 8}, {8, 16}, {12, 24}}}, {{{4, 8}, {8, 16}, {12, 
        24}}, {{8, 16}, {16, 32}, {24, 48}}}, {{{6, 12}, {12, 24}, {18, 
        36}}, {{12, 24}, {24, 48}, {36, 72}}}}}]]], "Output"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {"1"},
            {"2"},
            {"3"},
            {"4"},
            {"5"},
            {"6"},
            {"7"},
            {"8"},
            {"9"},
            {"10"}
            }], ")"}],
      MatrixForm[ {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}]]], "Output"],

Cell[BoxData[
    TagBox["10",
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MPut["\<a\>", \ a]\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("a =\n\n     1     2     3     4\n     2     4     6     8\n     3     \
6     9    12"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MPut["\<b\>", \ b]\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("b =\n\n   1.0000 + 1.0000i   2.0000 + 2.0000i   3.0000 + 3.0000i   \
4.0000 + 4.0000i\n   1.0000 + 1.0000i   2.0000 + 2.0000i   3.0000 + 3.0000i   \
4.0000 + 4.0000i\n   1.0000 + 1.0000i   2.0000 + 2.0000i   3.0000 + 3.0000i   \
4.0000 + 4.0000i"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MPut["\<c\>", c]\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("c(:,:,1) =\n\n     1     2     3\n     2     4     6\n\n\nc(:,:,2) =\n\
\n     2     4     6\n     4     8    12\n\n\nc(:,:,3) =\n\n     3     6     \
9\n     6    12    18\n\n\nc(:,:,4) =\n\n     4     8    12\n     8    16    \
24"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MPut["\<d\>", \ d]\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("d(:,:,1) =\n\n   2.0000 + 1.0000i   3.0000 + 1.0000i   4.0000 + \
1.0000i   5.0000 + 1.0000i\n   3.0000 + 1.0000i   4.0000 + 1.0000i   5.0000 + \
1.0000i   6.0000 + 1.0000i\n   4.0000 + 1.0000i   5.0000 + 1.0000i   6.0000 + \
1.0000i   7.0000 + 1.0000i\n   5.0000 + 1.0000i   6.0000 + 1.0000i   7.0000 + \
1.0000i   8.0000 + 1.0000i\n   6.0000 + 1.0000i   7.0000 + 1.0000i   8.0000 + \
1.0000i   9.0000 + 1.0000i\n\n\nd(:,:,2) =\n\n   2.0000 + 2.0000i   3.0000 + \
2.0000i   4.0000 + 2.0000i   5.0000 + 2.0000i\n   3.0000 + 2.0000i   4.0000 + \
2.0000i   5.0000 + 2.0000i   6.0000 + 2.0000i\n   4.0000 + 2.0000i   5.0000 + \
2.0000i   6.0000 + 2.0000i   7.0000 + 2.0000i\n   5.0000 + 2.0000i   6.0000 + \
2.0000i   7.0000 + 2.0000i   8.0000 + 2.0000i\n   6.0000 + 2.0000i   7.0000 + \
2.0000i   8.0000 + 2.0000i   9.0000 + 2.0000i"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MPut["\<e\>", \ e]\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("e(:,:,1,1,1) =\n\n     1     2     3\n     2     4     6\n\n\n\
e(:,:,2,1,1) =\n\n     2     4     6\n     4     8    12\n\n\ne(:,:,1,2,1) =\n\
\n     2     4     6\n     4     8    12\n\n\ne(:,:,2,2,1) =\n\n     4     8  \
  12\n     8    16    24\n\n\ne(:,:,1,3,1) =\n\n     3     6     9\n     6    \
12    18\n\n\ne(:,:,2,3,1) =\n\n     6    12    18\n    12    24    36\n\n\n\
e(:,:,1,1,2) =\n\n     2     4     6\n     4     8    12\n\n\ne(:,:,2,1,2) =\n\
\n     4     8    12\n     8    16    24\n\n\ne(:,:,1,2,2) =\n\n     4     8  \
  12\n     8    16    24\n\n\ne(:,:,2,2,2) =\n\n     8    16    24\n    16    \
32    48\n\n\ne(:,:,1,3,2) =\n\n     6    12    18\n    12    24    36\n\n\n\
e(:,:,2,3,2) =\n\n    12    24    36\n    24    48    72"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MPut["\<f\>", \ f]\)], "Input"],

Cell[BoxData[
    \("f =\n\n     1\n     2\n     3\n     4\n     5\n     6\n     7\n     8\n\
     9\n    10"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MPut["\<g\>", \ g]\)], "Input"],

Cell[BoxData[
    \("g =\n\n    10"\)], "Output"]
}, Open  ]],

Cell["\<\
It is also possible to get these matrices back.  We can translate a Matlab \
matrix to a Mathematica expression and compare the information when it is \
returned:\
\>", "Text",
  CellTags->"M"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(MGet[\ "\<a\>"\ ]\), 
    \(Dimensions[MGet["\<a\>"]]\), 
    \(a\), 
    \(Dimensions[a]\)}], "Input",
  CellTags->"M"],

Cell[BoxData[
    \({{1.`, 2.`, 3.`, 4.`}, {2.`, 4.`, 6.`, 8.`}, {3.`, 6.`, 9.`, 
        12.0000000000000017`}}\)], "Output"],

Cell[BoxData[
    \({3, 4}\)], "Output"],

Cell[BoxData[
    \({{1, 2, 3, 4}, {2, 4, 6, 8}, {3, 6, 9, 12}}\)], "Output"],

Cell[BoxData[
    \({3, 4}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(MGet[\ "\<b\>"\ ]\), 
    \(Dimensions[MGet["\<b\>"]]\), 
    \(b\), 
    \(Dimensions[b]\)}], "Input",
  CellTags->"M"],

Cell[BoxData[
    \({{\(1.`\[InvisibleSpace]\) + 1.`\ I, \(2.`\[InvisibleSpace]\) + 2.`\ I, 
        \(3.`\[InvisibleSpace]\) + 3.`\ I, 
        \(4.`\[InvisibleSpace]\) + 4.`\ I}, {
        \(1.`\[InvisibleSpace]\) + 1.`\ I, \(2.`\[InvisibleSpace]\) + 2.`\ I, 
        \(3.`\[InvisibleSpace]\) + 3.`\ I, 
        \(4.`\[InvisibleSpace]\) + 4.`\ I}, {
        \(1.`\[InvisibleSpace]\) + 1.`\ I, \(2.`\[InvisibleSpace]\) + 2.`\ I, 
        \(3.`\[InvisibleSpace]\) + 3.`\ I, \(4.`\[InvisibleSpace]\) + 4.`\ I}}
      \)], "Output"],

Cell[BoxData[
    \({3, 4}\)], "Output"],

Cell[BoxData[
    \({{1 + I, 2 + 2\ I, 3 + 3\ I, 4 + 4\ I}, {1 + I, 2 + 2\ I, 3 + 3\ I, 
        4 + 4\ I}, {1 + I, 2 + 2\ I, 3 + 3\ I, 4 + 4\ I}}\)], "Output"],

Cell[BoxData[
    \({3, 4}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(MGet[\ "\<c\>"\ ]\), 
    \(Dimensions[MGet["\<c\>"]]\), 
    \(c\), 
    \(Dimensions[c]\)}], "Input",
  CellTags->"M"],

Cell[BoxData[
    \({{{1.`, 2.`, 3.`, 4.`}, {2.`, 4.`, 6.`, 8.`}, {3.`, 6.`, 9.`, 
          12.0000000000000017`}}, {{2.`, 4.`, 6.`, 8.`}, {4.`, 8.`, 
          12.0000000000000017`, 16.0000000000000008`}, {6.`, 
          12.0000000000000017`, 18.`, 24.0000000000000035`}}}\)], "Output"],

Cell[BoxData[
    \({2, 3, 4}\)], "Output"],

Cell[BoxData[
    \({{{1, 2, 3, 4}, {2, 4, 6, 8}, {3, 6, 9, 12}}, {{2, 4, 6, 8}, {4, 8, 12, 
          16}, {6, 12, 18, 24}}}\)], "Output"],

Cell[BoxData[
    \({2, 3, 4}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(MGet[\ "\<d\>"\ ]\), 
    \(Dimensions[MGet["\<d\>"]]\), 
    \(d\), 
    \(Dimensions[d]\)}], "Input",
  CellTags->"M"],

Cell[BoxData[
    \({{{\(2.`\[InvisibleSpace]\) + 1.`\ I, 
          \(2.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(3.`\[InvisibleSpace]\) + 1.`\ I, 
          \(3.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(4.`\[InvisibleSpace]\) + 1.`\ I, 
          \(4.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(5.`\[InvisibleSpace]\) + 1.`\ I, 
          \(5.`\[InvisibleSpace]\) + 2.`\ I}}, {{
          \(3.`\[InvisibleSpace]\) + 1.`\ I, 
          \(3.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(4.`\[InvisibleSpace]\) + 1.`\ I, 
          \(4.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(5.`\[InvisibleSpace]\) + 1.`\ I, 
          \(5.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(6.`\[InvisibleSpace]\) + 1.`\ I, 
          \(6.`\[InvisibleSpace]\) + 2.`\ I}}, {{
          \(4.`\[InvisibleSpace]\) + 1.`\ I, 
          \(4.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(5.`\[InvisibleSpace]\) + 1.`\ I, 
          \(5.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(6.`\[InvisibleSpace]\) + 1.`\ I, 
          \(6.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(7.`\[InvisibleSpace]\) + 1.`\ I, 
          \(7.`\[InvisibleSpace]\) + 2.`\ I}}, {{
          \(5.`\[InvisibleSpace]\) + 1.`\ I, 
          \(5.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(6.`\[InvisibleSpace]\) + 1.`\ I, 
          \(6.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(7.`\[InvisibleSpace]\) + 1.`\ I, 
          \(7.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(8.`\[InvisibleSpace]\) + 1.`\ I, 
          \(8.`\[InvisibleSpace]\) + 2.`\ I}}, {{
          \(6.`\[InvisibleSpace]\) + 1.`\ I, 
          \(6.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(7.`\[InvisibleSpace]\) + 1.`\ I, 
          \(7.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(8.`\[InvisibleSpace]\) + 1.`\ I, 
          \(8.`\[InvisibleSpace]\) + 2.`\ I}, {
          \(9.`\[InvisibleSpace]\) + 1.`\ I, 
          \(9.`\[InvisibleSpace]\) + 2.`\ I}}}\)], "Output"],

Cell[BoxData[
    \({5, 4, 2}\)], "Output"],

Cell[BoxData[
    \({{{2 + I, 2 + 2\ I}, {3 + I, 3 + 2\ I}, {4 + I, 4 + 2\ I}, {5 + I, 
          5 + 2\ I}}, {{3 + I, 3 + 2\ I}, {4 + I, 4 + 2\ I}, {5 + I, 
          5 + 2\ I}, {6 + I, 6 + 2\ I}}, {{4 + I, 4 + 2\ I}, {5 + I, 
          5 + 2\ I}, {6 + I, 6 + 2\ I}, {7 + I, 7 + 2\ I}}, {{5 + I, 
          5 + 2\ I}, {6 + I, 6 + 2\ I}, {7 + I, 7 + 2\ I}, {8 + I, 
          8 + 2\ I}}, {{6 + I, 6 + 2\ I}, {7 + I, 7 + 2\ I}, {8 + I, 
          8 + 2\ I}, {9 + I, 9 + 2\ I}}}\)], "Output"],

Cell[BoxData[
    \({5, 4, 2}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(MGet[\ "\<e\>"\ ]\), 
    \(Dimensions[MGet["\<e\>"]]\), 
    \(e\), 
    \(Dimensions[e]\)}], "Input",
  CellTags->"M"],

Cell[BoxData[
    \({{{{{1.`, 2.`}, {2.`, 4.`}, {3.`, 6.`}}, {{2.`, 4.`}, {4.`, 8.`}, {6.`, 
              12.0000000000000017`}}}, {{{2.`, 4.`}, {4.`, 8.`}, {6.`, 
              12.0000000000000017`}}, {{4.`, 8.`}, {8.`, 
              16.0000000000000008`}, {12.0000000000000017`, 
              24.0000000000000035`}}}, {{{3.`, 6.`}, {6.`, 
              12.0000000000000017`}, {9.`, 18.`}}, {{6.`, 
              12.0000000000000017`}, {12.0000000000000017`, 
              24.0000000000000035`}, {18.`, 36.0000000000000008`}}}}, {{{{
              2.`, 4.`}, {4.`, 8.`}, {6.`, 12.0000000000000017`}}, {{4.`, 
              8.`}, {8.`, 16.0000000000000008`}, {12.0000000000000017`, 
              24.0000000000000035`}}}, {{{4.`, 8.`}, {8.`, 
              16.0000000000000008`}, {12.0000000000000017`, 
              24.0000000000000035`}}, {{8.`, 16.0000000000000008`}, {
              16.0000000000000008`, 32.0000000000000017`}, {
              24.0000000000000035`, 48.0000000000000071`}}}, {{{6.`, 
              12.0000000000000017`}, {12.0000000000000017`, 
              24.0000000000000035`}, {18.`, 36.0000000000000008`}}, {{
              12.0000000000000017`, 24.0000000000000035`}, {
              24.0000000000000035`, 48.0000000000000071`}, {
              36.0000000000000008`, 72.0000000000000017`}}}}}\)], "Output"],

Cell[BoxData[
    \({2, 3, 2, 3, 2}\)], "Output"],

Cell[BoxData[
    \({{{{{1, 2}, {2, 4}, {3, 6}}, {{2, 4}, {4, 8}, {6, 12}}}, {{{2, 4}, {4, 
              8}, {6, 12}}, {{4, 8}, {8, 16}, {12, 24}}}, {{{3, 6}, {6, 12}, {
              9, 18}}, {{6, 12}, {12, 24}, {18, 36}}}}, {{{{2, 4}, {4, 8}, {
              6, 12}}, {{4, 8}, {8, 16}, {12, 24}}}, {{{4, 8}, {8, 16}, {12, 
              24}}, {{8, 16}, {16, 32}, {24, 48}}}, {{{6, 12}, {12, 24}, {18, 
              36}}, {{12, 24}, {24, 48}, {36, 72}}}}}\)], "Output"],

Cell[BoxData[
    \({2, 3, 2, 3, 2}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(MGet[\ "\<f\>"\ ]\), 
    \(Dimensions[MGet["\<f\>"]]\), 
    \(f\), 
    \(Dimensions[f]\)}], "Input"],

Cell[BoxData[
    \({{1.`}, {2.`}, {3.`}, {4.`}, {5.`}, {6.`}, {7.`}, {8.`}, {9.`}, {10.`}}
      \)], "Output"],

Cell[BoxData[
    \({10, 1}\)], "Output"],

Cell[BoxData[
    \({1, 2, 3, 4, 5, 6, 7, 8, 9, 10}\)], "Output"],

Cell[BoxData[
    \({10}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(MGet["\<g\>"]\), 
    \(Dimensions[MGet["\<g\>"]]\), 
    \(g\), 
    \(Dimensions[g]\)}], "Input"],

Cell[BoxData[
    \(10.`\)], "Output"],

Cell[BoxData[
    \({}\)], "Output"],

Cell[BoxData[
    \(10\)], "Output"],

Cell[BoxData[
    \({}\)], "Output"]
}, Open  ]],

Cell["Start a Matlab mode", "Text",
  CellTags->"M"],

Cell[BoxData[
    \(MBegin[]\)], "Input",
  CellTags->"M"],

Cell["Every string is sent to the Matlab evaluator", "Text",
  CellTags->"M"],

Cell[CellGroupData[{

Cell[BoxData[
    \("\<m = rand(3,3)\>"\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("m =\n\n    0.9501    0.4860    0.4565\n    0.2311    0.8913    0.0185\n\
    0.6068    0.7621    0.8214"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \("\<inv(m)\>"\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("ans =\n\n    1.6740   -0.1196   -0.9276\n   -0.4165    1.1738    \
0.2050\n   -0.8504   -1.0006    1.7125"\)], "Output"]
}, Open  ]],

Cell["Return to the standard Mathematica evaluator", "Text",
  CellTags->"M"],

Cell[BoxData[
    \(MEnd[]\)], "Input",
  CellTags->"M"],

Cell["I.e. strings are no longer evaluated by Matlab ...", "Text",
  CellTags->"M"],

Cell[CellGroupData[{

Cell[BoxData[
    \("\<m = [1,2;3,4]\>"\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("m = [1,2;3,4]"\)], "Output"]
}, Open  ]],

Cell["... unless you really want them to!", "Text",
  CellTags->"M"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\ M[\ %\ ]\)\)], "Input",
  CellTags->"M"],

Cell[BoxData[
    \("m =\n\n     1     2\n     3     4"\)], "Output"]
}, Open  ]],

Cell[BoxData[{
    \(MClose[]\), 
    \(\(MatlabUninstall[]; \)\)}], "Input",
  CellTags->"M"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Notes - Read the fine print", "Subtitle",
  CellTags->"Notes"],

Cell[TextData[{
  "This package originally consisted of two seperate ",
  StyleBox["MathSource",
    FontSlant->"Italic"],
  " items -- Matlabio and MaMa.  They were updated to be Matlab 5.x \
compatible and combined into one all-inclusive Matlab package."
}], "Text",
  CellTags->"Notes"],

Cell[TextData[{
  "The code only deals with Reals (double type), so any arrays you create or \
move around will be converted.  This can easily be fixed in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " if you are sure that they are supposed to be Integers:"
}], "Text",
  CellTags->"Notes"],

Cell[BoxData[
    \(Map[Floor, \ ListToConvert]\)], "Input",
  CellTags->"Notes"],

Cell[TextData[{
  "Launching the engine is not possible from the Mac.  This is due The \
Mathworks' lack of engine libraries for the Mac.  However, it may be possible \
to access the engine capabilities from  another machine (UNIX or PC), since ",
  
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " supports TCP/IP connections to remote machines."
}], "Text",
  CellTags->"Notes"],

Cell[TextData[{
  "While multidimensional arrays are not directly supported through this \
interface, a bit of trickery allows users to pass them between the Matlab \
Engine and ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s Kernel.  Here are the functions which are used:"
}], "Text",
  CellTags->"Notes"],

Cell[BoxData[{
    \(MPut[name_String, \ x_List]\  := \ 
      Module[{dim}, \ 
        IntMPutMatrix[name, \ 
          Flatten@Transpose[x, \ 
              Reverse@\(Range@\(Length@\(Dimensions@x\)\)\)]]; \ 
        dim\  = \ 
          If[Length@\(Dimensions@x\)\  == \ 1, \ Append[Dimensions@x, \ 1], \ 
            Dimensions@x]; \ 
        M@\(StringJoin@{name, \ "\< = reshape(\>", \ name, \ "\<, \>", \ 
              StrDropLast@\(StrDropFirst@\(ToString@dim\)\), \ "\<)\>"}\)\ ]\n
    \), 
    \(MGet[name_String]\ \ \ \ \ \ \ \ \  := \ 
      Module[{mgpdims, \ mgpm}, \ 
        M@\(StringJoin@{"\<mgpdims = size(\>", \ name, \ "\<)\>"}\); \ 
        mgpdims\  = \ 
          Reverse@\(Floor@\(First@\(IntMGetMatrix@"\<mgpdims\>"\)\)\); \ 
        M@\(StringJoin@{"\<mgpm = reshape(\>", \ name, \ "\<,1,\>", \ 
              ToString@\(Floor@\((Times@@mgpdims)\)\), \ "\<)\>"}\); \ 
        mgpm\  = \ First@\(Matlab`Private`IntMGetMatrix@"\<mgpm\>"\); \ 
        M@"\<clear mgpdims mgpm\>"; \ 
        If[Floor@\((Times@@mgpdims)\) == \ 1, \ First@mgpm, \ 
          Transpose[First@Fold[Partition, \ mgpm, \ \ Reverse@mgpdims], \ 
            Reverse@\(Range@\(Length@mgpdims\)\)]]\ ]\)}], "Input"],

Cell["\<\
These functions are defined in the source code (mmp.tm) and use the \
programming facilities at both ends of the link to reshape or Partition one \
dimensional arrays into multidimensional matrices.  This is currently not \
supported for the MAT-File facilities, but a similar strategy could be \
used--Saving both the dimensions and the flattened (reshaped) list.\
\>", "Text",
  CellTags->"Notes"],

Cell["\<\
In order for these programs to work correctly, they directory structure \
created when you decompressed them must be placed in the AddOns/Applications \
directory or you must change all of the code that deals with these \
directories to your preferred directory.  It may also be necessary to move \
the following files from your Matlab directory to the \
/Addons/Applications/Matlab/Bin directory:\
\>", "Text",
  CellTags->"Notes"],

Cell["\<\
libeng.dll
libmat.dll
libmx.dll
libut.dll\
\>", "Text",
  FontWeight->"Bold",
  CellTags->"Notes"],

Cell["\<\
Alternatively, for Windows, you can add the Matlab Bin directory to your path \
(e.g. C:\\Program Files\\Matlab\\Bin) in your autoexec.bat file.  This will \
require a reboot, but save hard drive space. \
\>", "Text",
  CellTags->"Notes"],

Cell["\<\
It may be necessary to perform a similar operation under UNIX.  Move the lib \
files (.so) to the /usr/lib directory so that they can be found by the \
program at run-time. \
\>", "Text",
  CellTags->"Notes"],

Cell[CellGroupData[{

Cell["Compiling", "Subsection",
  CellTags->"Notes"],

Cell[CellGroupData[{

Cell["Under Windows", "Subsubsection",
  CellTags->"Notes"],

Cell["\<\
I prefer to create projects that are independent of Matlab's scripts.  It's \
easy enough to do.  You will need to move the .h files from your \
/extern/include directory to the include directory of your compile\
\>", "Text",
  CellTags->"Notes"],

Cell["\<\
You next need to create the lib files necessary for your projects.  Put the \
following files in a temporary directory (they are in /extern/include):\
\>", "Text",
  CellTags->"Notes"],

Cell["\<\
libeng.def
libmx.def
libmat.def\
\>", "Text",
  CellTags->"Notes"],

Cell["\<\
For MSVC you will need to run the following lines to produce the libs you \
need:\
\>", "Text",
  CellTags->"Notes"],

Cell["\<\
lib /def:libmx.def  /machine:ix86 /OUT:libmx.lib /NOLOGO
lib /def:libeng.def /machine:ix86 /OUT:libeng.lib /NOLOGO
lib /def:libmat.def /machine:ix86 /OUT:libmat.lib /NOLOGO\
\>", "Text",
  CellTags->"Notes"],

Cell["Put these in your compiler's /lib directory.", "Text",
  CellTags->"Notes"],

Cell[TextData[{
  "Now, you can create a project as normal, include the libs you need and \
compile.  See the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " documenation for more information about making your project work with ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->"Notes"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Under Unix", "Subsubsection",
  CellTags->"Notes"],

Cell["\<\
You can follow the directions normally after you have mprep'ed the .tm files.\
\
\>", "Text",
  CellTags->"Notes"],

Cell["mprep mmp.tm > mmp.c", "Text",
  CellTags->"Notes"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Under Mac", "Subsubsection",
  CellTags->"Notes"],

Cell["\<\
It should be possible to compile the MAT-file interface part of the program \
on the Mac with little difficulty, though I never did.  The Engine interface \
wouldn't be possible since the Engine Libraries aren't supported on the Mac \
at the time of this writing.\
\>", "Text",
  CellTags->"Notes"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{484, 234},
WindowMargins->{{2, Automatic}, {Automatic, 5}},
ShowCellTags->True,
StyleDefinitions -> "Default.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Introduction"->{
    Cell[1731, 51, 83, 1, 164, "Title",
      CellTags->"Introduction"],
    Cell[1817, 54, 119, 4, 66, "Text",
      CellTags->"Introduction"],
    Cell[1939, 60, 889, 25, 162, "Text",
      CellTags->"Introduction"]},
  "Installation"->{
    Cell[2853, 89, 60, 1, 78, "Subtitle",
      CellTags->"Installation"],
    Cell[2938, 94, 77, 1, 67, "Section",
      CellTags->"Installation"],
    Cell[3018, 97, 449, 8, 85, "Text",
      CellTags->"Installation"],
    Cell[3470, 107, 115, 7, 104, "Text",
      CellTags->"Installation"],
    Cell[3588, 116, 215, 5, 47, "Text",
      CellTags->"Installation"],
    Cell[3806, 123, 225, 5, 47, "Text",
      CellTags->"Installation"],
    Cell[4034, 130, 101, 1, 47, "Text",
      CellTags->"Installation"],
    Cell[4172, 136, 116, 3, 47, "Section",
      CellTags->"Installation"],
    Cell[4291, 141, 326, 10, 47, "Text",
      CellTags->"Installation"],
    Cell[4654, 156, 109, 2, 67, "Section",
      Evaluatable->False,
      CellTags->"Installation"],
    Cell[4766, 160, 94, 1, 47, "Text",
      CellTags->"Installation"],
    Cell[4863, 163, 78, 2, 44, "Input",
      CellTags->"Installation"],
    Cell[4944, 167, 658, 22, 47, "Text",
      Evaluatable->False,
      CellTags->"Installation"],
    Cell[5605, 191, 63, 1, 44, "Input",
      CellTags->"Installation"],
    Cell[5705, 197, 55, 1, 67, "Section",
      CellTags->"Installation"],
    Cell[5785, 202, 67, 1, 61, "Subsection",
      CellTags->"Installation"],
    Cell[5877, 207, 75, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[5955, 210, 149, 4, 47, "Text",
      CellTags->"Installation"],
    Cell[6141, 219, 86, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[6230, 222, 121, 3, 47, "Text",
      CellTags->"Installation"],
    Cell[6388, 230, 92, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[6483, 233, 244, 5, 47, "Text",
      CellTags->"Installation"],
    Cell[6764, 243, 93, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[6860, 246, 233, 5, 47, "Text",
      CellTags->"Installation"],
    Cell[7142, 257, 65, 1, 61, "Subsection",
      CellTags->"Installation"],
    Cell[7232, 262, 60, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[7295, 265, 80, 1, 47, "Text",
      CellTags->"Installation"],
    Cell[7412, 271, 61, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[7476, 274, 83, 1, 47, "Text",
      CellTags->"Installation"],
    Cell[7596, 280, 75, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[7674, 283, 102, 1, 47, "Text",
      CellTags->"Installation"],
    Cell[7813, 289, 61, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[7877, 292, 125, 3, 47, "Text",
      CellTags->"Installation"],
    Cell[8039, 300, 59, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[8101, 303, 64, 1, 47, "Text",
      CellTags->"Installation"],
    Cell[8202, 309, 73, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[8278, 312, 223, 5, 47, "Text",
      CellTags->"Installation"],
    Cell[8538, 322, 67, 1, 57, "Subsubsection",
      CellTags->"Installation"],
    Cell[8608, 325, 179, 4, 47, "Text",
      CellTags->"Installation"]},
  "Mat"->{
    Cell[8860, 337, 85, 1, 96, "Subtitle",
      CellTags->"Mat"],
    Cell[8970, 342, 79, 2, 67, "Section",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[9052, 346, 69, 2, 44, "Input",
      CellTags->"Mat"],
    Cell[9124, 350, 189, 4, 64, "Input",
      CellTags->"Mat"],
    Cell[9316, 356, 458, 15, 67, "Text",
      CellTags->"Mat"],
    Cell[9799, 375, 77, 2, 44, "Input",
      CellTags->"Mat"],
    Cell[9879, 379, 58, 2, 43, "Output",
      CellTags->"Mat"],
    Cell[9952, 384, 285, 6, 66, "Text",
      CellTags->"Mat"],
    Cell[10240, 392, 83, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[10348, 397, 53, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[10404, 400, 63, 2, 43, "Output",
      CellTags->"Mat"],
    Cell[10516, 408, 78, 2, 67, "Section",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[10597, 412, 111, 2, 47, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[10711, 416, 72, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[10786, 419, 59, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[10848, 422, 232, 6, 58, "SmallText",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[11083, 430, 67, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[11187, 436, 89, 2, 67, "Section",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[11279, 440, 445, 15, 85, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[11727, 457, 163, 7, 116, "Input",
      CellTags->"Mat"],
    Cell[11893, 466, 532, 10, 104, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[12428, 478, 110, 4, 62, "Input",
      CellTags->"Mat"],
    Cell[12541, 484, 105, 4, 62, "Input",
      CellTags->"Mat"],
    Cell[12649, 490, 97, 4, 62, "Input",
      CellTags->"Mat"],
    Cell[12783, 499, 79, 2, 67, "Section",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[12865, 503, 957, 30, 124, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[13825, 535, 140, 4, 80, "Input",
      CellTags->"Mat"],
    Cell[13968, 541, 74, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[14045, 544, 84, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[14166, 550, 85, 2, 67, "Section",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[14254, 554, 507, 18, 66, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[14764, 574, 102, 2, 62, "Input",
      CellTags->"Mat"],
    Cell[14891, 580, 84, 2, 44, "Input",
      CellTags->"Mat"],
    Cell[14978, 584, 69, 2, 43, "Output",
      CellTags->"Mat"],
    Cell[15096, 592, 99, 3, 67, "Section",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[15220, 599, 108, 2, 61, "Subsection",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[15331, 603, 984, 32, 106, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[16318, 637, 287, 6, 85, "Text",
      CellTags->"Mat"],
    Cell[16608, 645, 279, 8, 170, "Input",
      CellTags->"Mat"],
    Cell[16912, 657, 73, 2, 57, "Subsubsection",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[16988, 661, 93, 2, 47, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[17084, 665, 174, 7, 116, "Input",
      CellTags->"Mat"],
    Cell[17261, 674, 99, 2, 47, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[17385, 680, 60, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[17448, 683, 83, 2, 43, "Output",
      CellTags->"Mat"],
    Cell[17546, 688, 76, 2, 47, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[17625, 692, 84, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[17734, 697, 84, 2, 44, "Input",
      CellTags->"Mat"],
    Cell[17821, 701, 77, 2, 43, "Output",
      CellTags->"Mat"],
    Cell[17959, 710, 97, 2, 61, "Subsection",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[18059, 714, 621, 21, 67, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[18683, 737, 160, 7, 116, "Input",
      CellTags->"Mat"],
    Cell[18868, 748, 73, 2, 57, "Subsubsection",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[18944, 752, 97, 2, 47, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[19044, 756, 87, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[19134, 759, 650, 22, 87, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[19809, 785, 121, 6, 116, "Input",
      CellTags->"Mat"],
    Cell[19933, 793, 505, 15, 62, "Output",
      CellTags->"Mat"],
    Cell[20453, 811, 389, 14, 48, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[20867, 829, 62, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[20932, 832, 154, 4, 43, "Output",
      CellTags->"Mat"],
    Cell[21159, 844, 100, 3, 67, "Section",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[21262, 849, 572, 20, 66, "Text",
      Evaluatable->False,
      CellTags->"Mat"],
    Cell[21859, 873, 60, 1, 44, "Input",
      CellTags->"Mat"],
    Cell[21922, 876, 151, 4, 58, "Message",
      CellTags->"Mat"],
    Cell[22076, 882, 60, 2, 43, "Output",
      CellTags->"Mat"],
    Cell[22151, 887, 54, 1, 44, "Input",
      CellTags->"Mat"]},
  "M"->{
    Cell[22254, 894, 68, 1, 67, "Subtitle",
      CellTags->"M"],
    Cell[22325, 897, 1152, 35, 180, "Text",
      CellTags->"M"],
    Cell[23502, 936, 43, 1, 67, "Section",
      CellTags->"M"],
    Cell[23548, 939, 50, 1, 47, "Text",
      CellTags->"M"],
    Cell[23601, 942, 67, 2, 44, "Input",
      CellTags->"M"],
    Cell[23671, 946, 63, 1, 47, "Text",
      CellTags->"M"],
    Cell[23737, 949, 57, 2, 44, "Input",
      CellTags->"M"],
    Cell[23797, 953, 77, 1, 47, "Text",
      CellTags->"M"],
    Cell[23899, 958, 76, 2, 44, "Input",
      CellTags->"M"],
    Cell[24064, 966, 260, 7, 47, "Text",
      CellTags->"M"],
    Cell[24349, 977, 85, 2, 44, "Input",
      CellTags->"M"],
    Cell[24523, 985, 164, 6, 47, "Text",
      CellTags->"M"],
    Cell[24712, 995, 77, 2, 44, "Input",
      CellTags->"M"],
    Cell[24878, 1003, 215, 7, 66, "Text",
      CellTags->"M"],
    Cell[25118, 1014, 713, 15, 164, "Input",
      CellTags->"M"],
    Cell[39091, 1435, 68, 2, 44, "Input",
      CellTags->"M"],
    Cell[39320, 1446, 68, 2, 44, "Input",
      CellTags->"M"],
    Cell[39709, 1459, 66, 2, 44, "Input",
      CellTags->"M"],
    Cell[40083, 1472, 68, 2, 44, "Input",
      CellTags->"M"],
    Cell[41057, 1492, 68, 2, 44, "Input",
      CellTags->"M"],
    Cell[42289, 1528, 203, 5, 47, "Text",
      CellTags->"M"],
    Cell[42517, 1537, 141, 5, 104, "Input",
      CellTags->"M"],
    Cell[42990, 1560, 141, 5, 104, "Input",
      CellTags->"M"],
    Cell[43950, 1591, 141, 5, 104, "Input",
      CellTags->"M"],
    Cell[44654, 1617, 141, 5, 104, "Input",
      CellTags->"M"],
    Cell[47333, 1684, 141, 5, 104, "Input",
      CellTags->"M"],
    Cell[50153, 1771, 52, 1, 47, "Text",
      CellTags->"M"],
    Cell[50208, 1774, 58, 2, 44, "Input",
      CellTags->"M"],
    Cell[50269, 1778, 77, 1, 47, "Text",
      CellTags->"M"],
    Cell[50371, 1783, 69, 2, 44, "Input",
      CellTags->"M"],
    Cell[50619, 1794, 62, 2, 44, "Input",
      CellTags->"M"],
    Cell[50840, 1803, 77, 1, 47, "Text",
      CellTags->"M"],
    Cell[50920, 1806, 56, 2, 44, "Input",
      CellTags->"M"],
    Cell[50979, 1810, 83, 1, 47, "Text",
      CellTags->"M"],
    Cell[51087, 1815, 69, 2, 44, "Input",
      CellTags->"M"],
    Cell[51223, 1823, 68, 1, 47, "Text",
      CellTags->"M"],
    Cell[51316, 1828, 64, 2, 44, "Input",
      CellTags->"M"],
    Cell[51467, 1836, 94, 3, 64, "Input",
      CellTags->"M"]},
  "Notes"->{
    Cell[51610, 1845, 68, 1, 67, "Subtitle",
      CellTags->"Notes"],
    Cell[51681, 1848, 289, 7, 85, "Text",
      CellTags->"Notes"],
    Cell[51973, 1857, 302, 7, 85, "Text",
      CellTags->"Notes"],
    Cell[52278, 1866, 81, 2, 44, "Input",
      CellTags->"Notes"],
    Cell[52362, 1870, 385, 9, 104, "Text",
      CellTags->"Notes"],
    Cell[52750, 1881, 320, 8, 85, "Text",
      CellTags->"Notes"],
    Cell[54289, 1916, 409, 7, 123, "Text",
      CellTags->"Notes"],
    Cell[54701, 1925, 442, 8, 123, "Text",
      CellTags->"Notes"],
    Cell[55146, 1935, 108, 7, 104, "Text",
      CellTags->"Notes"],
    Cell[55257, 1944, 248, 5, 85, "Text",
      CellTags->"Notes"],
    Cell[55508, 1951, 218, 5, 66, "Text",
      CellTags->"Notes"],
    Cell[55751, 1960, 52, 1, 61, "Subsection",
      CellTags->"Notes"],
    Cell[55828, 1965, 59, 1, 57, "Subsubsection",
      CellTags->"Notes"],
    Cell[55890, 1968, 256, 5, 85, "Text",
      CellTags->"Notes"],
    Cell[56149, 1975, 194, 4, 66, "Text",
      CellTags->"Notes"],
    Cell[56346, 1981, 76, 5, 85, "Text",
      CellTags->"Notes"],
    Cell[56425, 1988, 126, 4, 47, "Text",
      CellTags->"Notes"],
    Cell[56554, 1994, 217, 5, 85, "Text",
      CellTags->"Notes"],
    Cell[56774, 2001, 81, 1, 47, "Text",
      CellTags->"Notes"],
    Cell[56858, 2004, 326, 10, 85, "Text",
      CellTags->"Notes"],
    Cell[57221, 2019, 56, 1, 57, "Subsubsection",
      CellTags->"Notes"],
    Cell[57280, 2022, 124, 4, 47, "Text",
      CellTags->"Notes"],
    Cell[57407, 2028, 57, 1, 47, "Text",
      CellTags->"Notes"],
    Cell[57501, 2034, 55, 1, 57, "Subsubsection",
      CellTags->"Notes"],
    Cell[57559, 2037, 308, 6, 104, "Text",
      CellTags->"Notes"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Introduction", 58575, 2067},
  {"Installation", 58815, 2074},
  {"Mat", 61833, 2155},
  {"M", 66954, 2316},
  {"Notes", 69400, 2393}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 83, 1, 164, "Title",
  CellTags->"Introduction"],
Cell[1817, 54, 119, 4, 66, "Text",
  CellTags->"Introduction"],
Cell[1939, 60, 889, 25, 162, "Text",
  CellTags->"Introduction"],

Cell[CellGroupData[{
Cell[2853, 89, 60, 1, 78, "Subtitle",
  CellTags->"Installation"],

Cell[CellGroupData[{
Cell[2938, 94, 77, 1, 67, "Section",
  CellTags->"Installation"],
Cell[3018, 97, 449, 8, 85, "Text",
  CellTags->"Installation"],
Cell[3470, 107, 115, 7, 104, "Text",
  CellTags->"Installation"],
Cell[3588, 116, 215, 5, 47, "Text",
  CellTags->"Installation"],
Cell[3806, 123, 225, 5, 47, "Text",
  CellTags->"Installation"],
Cell[4034, 130, 101, 1, 47, "Text",
  CellTags->"Installation"]
}, Closed]],

Cell[CellGroupData[{
Cell[4172, 136, 116, 3, 47, "Section",
  CellTags->"Installation"],
Cell[4291, 141, 326, 10, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4654, 156, 109, 2, 67, "Section",
  Evaluatable->False,
  CellTags->"Installation"],
Cell[4766, 160, 94, 1, 47, "Text",
  CellTags->"Installation"],
Cell[4863, 163, 78, 2, 44, "Input",
  CellTags->"Installation"],
Cell[4944, 167, 658, 22, 47, "Text",
  Evaluatable->False,
  CellTags->"Installation"],
Cell[5605, 191, 63, 1, 44, "Input",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5705, 197, 55, 1, 67, "Section",
  CellTags->"Installation"],

Cell[CellGroupData[{
Cell[5785, 202, 67, 1, 61, "Subsection",
  CellTags->"Installation"],

Cell[CellGroupData[{
Cell[5877, 207, 75, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[5955, 210, 149, 4, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6141, 219, 86, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[6230, 222, 121, 3, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6388, 230, 92, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[6483, 233, 244, 5, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6764, 243, 93, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[6860, 246, 233, 5, 47, "Text",
  CellTags->"Installation"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7142, 257, 65, 1, 61, "Subsection",
  CellTags->"Installation"],

Cell[CellGroupData[{
Cell[7232, 262, 60, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[7295, 265, 80, 1, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7412, 271, 61, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[7476, 274, 83, 1, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7596, 280, 75, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[7674, 283, 102, 1, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7813, 289, 61, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[7877, 292, 125, 3, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8039, 300, 59, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[8101, 303, 64, 1, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8202, 309, 73, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[8278, 312, 223, 5, 47, "Text",
  CellTags->"Installation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8538, 322, 67, 1, 57, "Subsubsection",
  CellTags->"Installation"],
Cell[8608, 325, 179, 4, 47, "Text",
  CellTags->"Installation"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[8860, 337, 85, 1, 96, "Subtitle",
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[8970, 342, 79, 2, 67, "Section",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[9052, 346, 69, 2, 44, "Input",
  CellTags->"Mat"],
Cell[9124, 350, 189, 4, 64, "Input",
  CellTags->"Mat"],
Cell[9316, 356, 458, 15, 67, "Text",
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[9799, 375, 77, 2, 44, "Input",
  CellTags->"Mat"],
Cell[9879, 379, 58, 2, 43, "Output",
  CellTags->"Mat"]
}, Open  ]],
Cell[9952, 384, 285, 6, 66, "Text",
  CellTags->"Mat"],
Cell[10240, 392, 83, 1, 44, "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[10348, 397, 53, 1, 44, "Input",
  CellTags->"Mat"],
Cell[10404, 400, 63, 2, 43, "Output",
  CellTags->"Mat"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10516, 408, 78, 2, 67, "Section",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[10597, 412, 111, 2, 47, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[10711, 416, 72, 1, 44, "Input",
  CellTags->"Mat"],
Cell[10786, 419, 59, 1, 44, "Input",
  CellTags->"Mat"],
Cell[10848, 422, 232, 6, 58, "SmallText",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[11083, 430, 67, 1, 44, "Input",
  CellTags->"Mat"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11187, 436, 89, 2, 67, "Section",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[11279, 440, 445, 15, 85, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[11727, 457, 163, 7, 116, "Input",
  CellTags->"Mat"],
Cell[11893, 466, 532, 10, 104, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[12428, 478, 110, 4, 62, "Input",
  CellTags->"Mat"],
Cell[12541, 484, 105, 4, 62, "Input",
  CellTags->"Mat"],
Cell[12649, 490, 97, 4, 62, "Input",
  CellTags->"Mat"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12783, 499, 79, 2, 67, "Section",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[12865, 503, 957, 30, 124, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[13825, 535, 140, 4, 80, "Input",
  CellTags->"Mat"],
Cell[13968, 541, 74, 1, 44, "Input",
  CellTags->"Mat"],
Cell[14045, 544, 84, 1, 44, "Input",
  CellTags->"Mat"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14166, 550, 85, 2, 67, "Section",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[14254, 554, 507, 18, 66, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[14764, 574, 102, 2, 62, "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[14891, 580, 84, 2, 44, "Input",
  CellTags->"Mat"],
Cell[14978, 584, 69, 2, 43, "Output",
  CellTags->"Mat"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[15096, 592, 99, 3, 67, "Section",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[15220, 599, 108, 2, 61, "Subsection",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[15331, 603, 984, 32, 106, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[16318, 637, 287, 6, 85, "Text",
  CellTags->"Mat"],
Cell[16608, 645, 279, 8, 170, "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[16912, 657, 73, 2, 57, "Subsubsection",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[16988, 661, 93, 2, 47, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[17084, 665, 174, 7, 116, "Input",
  CellTags->"Mat"],
Cell[17261, 674, 99, 2, 47, "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[17385, 680, 60, 1, 44, "Input",
  CellTags->"Mat"],
Cell[17448, 683, 83, 2, 43, "Output",
  CellTags->"Mat"]
}, Open  ]],
Cell[17546, 688, 76, 2, 47, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[17625, 692, 84, 1, 44, "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[17734, 697, 84, 2, 44, "Input",
  CellTags->"Mat"],
Cell[17821, 701, 77, 2, 43, "Output",
  CellTags->"Mat"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[17959, 710, 97, 2, 61, "Subsection",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[18059, 714, 621, 21, 67, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[18683, 737, 160, 7, 116, "Input",
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[18868, 748, 73, 2, 57, "Subsubsection",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[18944, 752, 97, 2, 47, "Text",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[19044, 756, 87, 1, 44, "Input",
  CellTags->"Mat"],
Cell[19134, 759, 650, 22, 87, "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[19809, 785, 121, 6, 116, "Input",
  CellTags->"Mat"],
Cell[19933, 793, 505, 15, 62, "Output",
  CellTags->"Mat"]
}, Open  ]],
Cell[20453, 811, 389, 14, 48, "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[20867, 829, 62, 1, 44, "Input",
  CellTags->"Mat"],
Cell[20932, 832, 154, 4, 43, "Output",
  CellTags->"Mat"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[21159, 844, 100, 3, 67, "Section",
  Evaluatable->False,
  CellTags->"Mat"],
Cell[21262, 849, 572, 20, 66, "Text",
  Evaluatable->False,
  CellTags->"Mat"],

Cell[CellGroupData[{
Cell[21859, 873, 60, 1, 44, "Input",
  CellTags->"Mat"],
Cell[21922, 876, 151, 4, 58, "Message",
  CellTags->"Mat"],
Cell[22076, 882, 60, 2, 43, "Output",
  CellTags->"Mat"]
}, Open  ]],
Cell[22151, 887, 54, 1, 44, "Input",
  CellTags->"Mat"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[22254, 894, 68, 1, 67, "Subtitle",
  CellTags->"M"],
Cell[22325, 897, 1152, 35, 180, "Text",
  CellTags->"M"],

Cell[CellGroupData[{
Cell[23502, 936, 43, 1, 67, "Section",
  CellTags->"M"],
Cell[23548, 939, 50, 1, 47, "Text",
  CellTags->"M"],
Cell[23601, 942, 67, 2, 44, "Input",
  CellTags->"M"],
Cell[23671, 946, 63, 1, 47, "Text",
  CellTags->"M"],
Cell[23737, 949, 57, 2, 44, "Input",
  CellTags->"M"],
Cell[23797, 953, 77, 1, 47, "Text",
  CellTags->"M"],

Cell[CellGroupData[{
Cell[23899, 958, 76, 2, 44, "Input",
  CellTags->"M"],
Cell[23978, 962, 71, 1, 86, "Output"]
}, Open  ]],
Cell[24064, 966, 260, 7, 47, "Text",
  CellTags->"M"],

Cell[CellGroupData[{
Cell[24349, 977, 85, 2, 44, "Input",
  CellTags->"M"],
Cell[24437, 981, 71, 1, 86, "Output"]
}, Open  ]],
Cell[24523, 985, 164, 6, 47, "Text",
  CellTags->"M"],

Cell[CellGroupData[{
Cell[24712, 995, 77, 2, 44, "Input",
  CellTags->"M"],
Cell[24792, 999, 71, 1, 86, "Output"]
}, Open  ]],
Cell[24878, 1003, 215, 7, 66, "Text",
  CellTags->"M"],

Cell[CellGroupData[{
Cell[25118, 1014, 713, 15, 164, "Input",
  CellTags->"M"],
Cell[25834, 1031, 298, 10, 56, "Output"],
Cell[26135, 1043, 396, 10, 56, "Output"],
Cell[26534, 1055, 1313, 43, 136, "Output"],
Cell[27850, 1100, 3980, 129, 168, "Output"],
Cell[31833, 1231, 6807, 179, 136, "Output"],
Cell[38643, 1412, 342, 14, 168, "Output"],
Cell[38988, 1428, 66, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39091, 1435, 68, 2, 44, "Input",
  CellTags->"M"],
Cell[39162, 1439, 121, 2, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39320, 1446, 68, 2, 44, "Input",
  CellTags->"M"],
Cell[39391, 1450, 281, 4, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39709, 1459, 66, 2, 44, "Input",
  CellTags->"M"],
Cell[39778, 1463, 268, 4, 428, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[40083, 1472, 68, 2, 44, "Input",
  CellTags->"M"],
Cell[40154, 1476, 866, 11, 314, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41057, 1492, 68, 2, 44, "Input",
  CellTags->"M"],
Cell[41128, 1496, 792, 10, 1340, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41957, 1511, 51, 1, 30, "Input"],
Cell[42011, 1514, 123, 2, 238, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42171, 1521, 51, 1, 30, "Input"],
Cell[42225, 1524, 49, 1, 67, "Output"]
}, Open  ]],
Cell[42289, 1528, 203, 5, 47, "Text",
  CellTags->"M"],

Cell[CellGroupData[{
Cell[42517, 1537, 141, 5, 104, "Input",
  CellTags->"M"],
Cell[42661, 1544, 126, 2, 29, "Output"],
Cell[42790, 1548, 40, 1, 29, "Output"],
Cell[42833, 1551, 77, 1, 29, "Output"],
Cell[42913, 1554, 40, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42990, 1560, 141, 5, 104, "Input",
  CellTags->"M"],
Cell[43134, 1567, 530, 9, 29, "Output"],
Cell[43667, 1578, 40, 1, 29, "Output"],
Cell[43710, 1581, 160, 2, 29, "Output"],
Cell[43873, 1585, 40, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[43950, 1591, 141, 5, 104, "Input",
  CellTags->"M"],
Cell[44094, 1598, 289, 4, 29, "Output"],
Cell[44386, 1604, 43, 1, 29, "Output"],
Cell[44432, 1607, 139, 2, 29, "Output"],
Cell[44574, 1611, 43, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44654, 1617, 141, 5, 104, "Input",
  CellTags->"M"],
Cell[44798, 1624, 1913, 40, 105, "Output"],
Cell[46714, 1666, 43, 1, 29, "Output"],
Cell[46760, 1669, 490, 7, 67, "Output"],
Cell[47253, 1678, 43, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47333, 1684, 141, 5, 104, "Input",
  CellTags->"M"],
Cell[47477, 1691, 1338, 20, 105, "Output"],
Cell[48818, 1713, 49, 1, 29, "Output"],
Cell[48870, 1716, 472, 6, 67, "Output"],
Cell[49345, 1724, 49, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49431, 1730, 124, 4, 90, "Input"],
Cell[49558, 1736, 112, 2, 29, "Output"],
Cell[49673, 1740, 41, 1, 29, "Output"],
Cell[49717, 1743, 65, 1, 29, "Output"],
Cell[49785, 1746, 38, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49860, 1752, 120, 4, 90, "Input"],
Cell[49983, 1758, 38, 1, 29, "Output"],
Cell[50024, 1761, 36, 1, 29, "Output"],
Cell[50063, 1764, 36, 1, 29, "Output"],
Cell[50102, 1767, 36, 1, 29, "Output"]
}, Open  ]],
Cell[50153, 1771, 52, 1, 47, "Text",
  CellTags->"M"],
Cell[50208, 1774, 58, 2, 44, "Input",
  CellTags->"M"],
Cell[50269, 1778, 77, 1, 47, "Text",
  CellTags->"M"],

Cell[CellGroupData[{
Cell[50371, 1783, 69, 2, 44, "Input",
  CellTags->"M"],
Cell[50443, 1787, 139, 2, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50619, 1794, 62, 2, 44, "Input",
  CellTags->"M"],
Cell[50684, 1798, 141, 2, 105, "Output"]
}, Open  ]],
Cell[50840, 1803, 77, 1, 47, "Text",
  CellTags->"M"],
Cell[50920, 1806, 56, 2, 44, "Input",
  CellTags->"M"],
Cell[50979, 1810, 83, 1, 47, "Text",
  CellTags->"M"],

Cell[CellGroupData[{
Cell[51087, 1815, 69, 2, 44, "Input",
  CellTags->"M"],
Cell[51159, 1819, 49, 1, 29, "Output"]
}, Open  ]],
Cell[51223, 1823, 68, 1, 47, "Text",
  CellTags->"M"],

Cell[CellGroupData[{
Cell[51316, 1828, 64, 2, 44, "Input",
  CellTags->"M"],
Cell[51383, 1832, 69, 1, 86, "Output"]
}, Open  ]],
Cell[51467, 1836, 94, 3, 64, "Input",
  CellTags->"M"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[51610, 1845, 68, 1, 67, "Subtitle",
  CellTags->"Notes"],
Cell[51681, 1848, 289, 7, 85, "Text",
  CellTags->"Notes"],
Cell[51973, 1857, 302, 7, 85, "Text",
  CellTags->"Notes"],
Cell[52278, 1866, 81, 2, 44, "Input",
  CellTags->"Notes"],
Cell[52362, 1870, 385, 9, 104, "Text",
  CellTags->"Notes"],
Cell[52750, 1881, 320, 8, 85, "Text",
  CellTags->"Notes"],
Cell[53073, 1891, 1213, 23, 370, "Input"],
Cell[54289, 1916, 409, 7, 123, "Text",
  CellTags->"Notes"],
Cell[54701, 1925, 442, 8, 123, "Text",
  CellTags->"Notes"],
Cell[55146, 1935, 108, 7, 104, "Text",
  CellTags->"Notes"],
Cell[55257, 1944, 248, 5, 85, "Text",
  CellTags->"Notes"],
Cell[55508, 1951, 218, 5, 66, "Text",
  CellTags->"Notes"],

Cell[CellGroupData[{
Cell[55751, 1960, 52, 1, 61, "Subsection",
  CellTags->"Notes"],

Cell[CellGroupData[{
Cell[55828, 1965, 59, 1, 57, "Subsubsection",
  CellTags->"Notes"],
Cell[55890, 1968, 256, 5, 85, "Text",
  CellTags->"Notes"],
Cell[56149, 1975, 194, 4, 66, "Text",
  CellTags->"Notes"],
Cell[56346, 1981, 76, 5, 85, "Text",
  CellTags->"Notes"],
Cell[56425, 1988, 126, 4, 47, "Text",
  CellTags->"Notes"],
Cell[56554, 1994, 217, 5, 85, "Text",
  CellTags->"Notes"],
Cell[56774, 2001, 81, 1, 47, "Text",
  CellTags->"Notes"],
Cell[56858, 2004, 326, 10, 85, "Text",
  CellTags->"Notes"]
}, Open  ]],

Cell[CellGroupData[{
Cell[57221, 2019, 56, 1, 57, "Subsubsection",
  CellTags->"Notes"],
Cell[57280, 2022, 124, 4, 47, "Text",
  CellTags->"Notes"],
Cell[57407, 2028, 57, 1, 47, "Text",
  CellTags->"Notes"]
}, Open  ]],

Cell[CellGroupData[{
Cell[57501, 2034, 55, 1, 57, "Subsubsection",
  CellTags->"Notes"],
Cell[57559, 2037, 308, 6, 104, "Text",
  CellTags->"Notes"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

